////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/CANopenAccess.cpp $
//
//	project   :  CAN - Controller Area Network
//	
//	purpose   :  CANopen Master API (using SocketCAN)
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenAccess
//	
//	includes  :  CANopenAccess.h (cop/default.h), cop/cop_api.h
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Master API (using SocketCAN)

static const char _id[] = "$Id: CANopenAccess.cpp 60 2009-11-07 21:47:27Z saturn $";
static const char _uv[] = "(C) 2005-2009 by UV Software, Friedrichshafen";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenAccess.h"
#include "cop/cop_api.h"
#include "tcp/tcp_open.h"

#include <string.h>

//////////////////////////////////////////////////////////////////////////////
//	defines

#define COPERR_LAST_ERROR	-32768		// last entry in error list

//////////////////////////////////////////////////////////////////////////////
//	error strings

static const struct {					// CAN errors:
	int   code;							//   error code
	char* text;							//   error text
}	error[] =
{
	{COPERR_NOERROR,					"No error occurred."},
	{COPERR_BOFF,						"Bus-off status."},
	{COPERR_EWRN,						"Error warning status."},
	{COPERR_BERR,						"Bus error."},
	{COPERR_OFFLINE,					"Not started yet."},
	{COPERR_ONLINE,						"Already started."},
	{COPERR_MSG_LST,					"Message lost (overrun)."},
	{COPERR_LEC_STUFF,					"Stuff error."},
	{COPERR_LEC_FORM,					"Form error."},
	{COPERR_LEC_ACK,					"Acknowledge error."},
	{COPERR_LEC_BIT1,					"Recessive bit error."},
	{COPERR_LEC_BIT0,					"Dominant bit error."},
	{COPERR_LEC_CRC,					"Checksum error."},
	{COPERR_TX_BUSY,					"Transmitter busy."},
	{COPERR_RX_EMPTY,					"Receiver empty."},
	{COPERR_QUE_OVR,					"Queue overrun."},
	{COPERR_TIMEOUT,					"Time-out error."},
	{COPERR_LENGTH,						"Invalid length."},
	{COPERR_FORMAT,						"Format error."},
	{COPERR_STATUS,						"Illegal status."},
	{COPERR_NODE_ID,					"Illegal node-id."},
	{COPERR_BAUDRATE,					"Illegal baud rate."},
	{COPERR_ILLPARA,					"Illegal parameter."},
	{COPERR_NULLPTR,					"Illegal parameter (NULL-Pointer)."},
	{COPERR_NOTINIT,					"Not initialized."},
	{COPERR_YETINIT,					"Already initialized."},
	//{COPERR_IDEXIST,					"Node already connected."},
	{COPERR_NOTSUPP,					"Function not supported!"},
	{COPERR_FATAL,						"Fatal error!"},
	{COPERR_ABORTED,					"Transfer aborted."},
	{COPERR_LAST_ERROR,					"Resistance is futile, prepare to assimilate!"}
};
static const struct {					// SDO Aborts:
	long  code;							//   SDO abort code
	char* text;							//   SDO abort text
}	abort[] =
{
	{0x05030000,						"Toggle bit not alternated."},
	{0x05040000,						"SDO protocol timed out."},
	{0x05040001,						"Client/server command specifier not valid or unknown."},
	{0x05040002,						"Invalid block size (block mode only)."},
	{0x05040003,						"Invalid sequence number (block mode only)."},
	{0x05040004,						"CRC error (block mode only)."},
	{0x05040005,						"Out of memory."},
	{0x06010000,						"Unsupported access to an object."},
	{0x06010001,						"Attempt to read a write only object."},
	{0x06010002,						"Attempt to write a read only object."},
	{0x06020000,						"Object does not exist in the object dictionary."},
	{0x06040041,						"Object cannot be mapped to the PDO."},
	{0x06040042,						"The number and length of the objects to be mapped would exceed PDO length."},
	{0x06040043,						"General parameter incompatibility reason."},
	{0x06040047,						"General internal incompatibility in the device."},
	{0x06060000,						"Access failed due to an hardware error."},
	{0x06070010,						"Data type does not match, length of service parameter does not match"},
	{0x06070012,						"Data type does not match, length of service parameter too high"},
	{0x06070013,						"Data type does not match, length of service parameter too low"},
	{0x06090011,						"Sub-index does not exist."},
	{0x06090030,						"Value range of parameter exceeded (only for write access)."},
	{0x06090031,						"Value of parameter written too high."},
	{0x06090032,						"Value of parameter written too low."},
	{0x06090036,						"Maximum value is less than minimum value."},
	{0x08000000,						"general error"},
	{0x08000020,						"Data cannot be transferred or stored to the application."},
	{0x08000021,						"Data cannot be transferred or stored to the application because of local control."},
	{0x08000022,						"Data cannot be transferred or stored to the application because of the present device state."},
	{0x08000023,						"Object dictionary dynamic generation fails or no object dictionary is present (e.g. object dictionary is generated from file and generation fails because of an file error)."},
	{0xFFFFFFFF,						"Unknown SDO abort code."}
};

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenAccess

CCANopenAccess::CCANopenAccess()
{
	// initialize attributes
	m_library   = CCANopenAccess::libUndefined;
	m_lastError = COPERR_NOERROR;
	m_lastAbort = 0x00000000L;
}

CCANopenAccess::~CCANopenAccess()
{
	// terminate CAN interface if initialized
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		cop_exit();
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		top_exit();
		break;
	default:
		break;
	}
}

bool CCANopenAccess::initialize( SHORT library, LONG board, LPVOID param, BYTE baudrate )
{
	struct _top_param    *top_param;
	struct _top_settings *top_settings;

	if( m_library != CCANopenAccess::libUndefined )
	{
		m_lastError = COPERR_YETINIT;
		return false;
	}
	switch( library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = cop_init( board, param, baudrate )) == COPERR_NOERROR)
		{
			m_library = library;
			return true;
		}
		else
		{
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( param )
		{
			top_param    = (struct _top_param*)param;
			top_settings = (struct _top_settings*)&top_param->settings;
		}
		else
		{
			m_lastError = COPERR_NULLPTR;
			return false;
		}
		if( (m_lastError = top_init( top_param->addr, top_param->port, top_settings, baudrate )) == COPERR_NOERROR)
		{
			m_library = library;
			return true;
		}
		else
		{
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTSUPP;
		return false;
	}
}

bool CCANopenAccess::terminate()
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = cop_exit()) == COPERR_NOERROR)
		{
			m_library = CCANopenAccess::libUndefined;
			return true;
		}
		else
		{
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_exit()) == COPERR_NOERROR)
		{
			m_library = CCANopenAccess::libUndefined;
			return true;
		}
		else
		{
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::reset( BYTE baudrate )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = cop_reset( baudrate )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_reset( baudrate )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::setSDOTimeout( WORD newValue, WORD* oldValue )
{
	WORD result;

	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		result = sdo_timeout( newValue );
		if( oldValue )
			*oldValue = result;
		return true;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_timeout( newValue, oldValue )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::write( BYTE nodeId, WORD index, BYTE subindex, SHORT length, BYTE* data )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_write( nodeId, index, subindex, length, data )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_write( nodeId, index, subindex, length, data )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::writeInteger8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER8 value )
{
	return writeUnsigned8( nodeId, index, subindex, (SDO_t_UNSIGNED8)value );
}

bool CCANopenAccess::writeInteger16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER16 value )
{
	return writeUnsigned16( nodeId, index, subindex, (SDO_t_UNSIGNED16)value );
}

bool CCANopenAccess::writeInteger32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER32 value )
{
	return writeUnsigned32( nodeId, index, subindex, (SDO_t_UNSIGNED32)value );
}

bool CCANopenAccess::writeUnsigned8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED8 value )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_write_8bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_write_8bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::writeUnsigned16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED16 value )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_write_16bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_write_16bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::writeUnsigned32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED32 value )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_write_32bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_write_32bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::read( BYTE nodeId, WORD index, BYTE subindex, SHORT* length, BYTE* data, SHORT size )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_read( nodeId, index, subindex, length, data, size )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_read( nodeId, index, subindex, length, data, size )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::readInteger8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER8* value )
{
	return readUnsigned8( nodeId, index, subindex, (SDO_t_UNSIGNED8*)value );
}

bool CCANopenAccess::readInteger16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER16* value )
{
	return readUnsigned16( nodeId, index, subindex, (SDO_t_UNSIGNED16*)value );
}

bool CCANopenAccess::readInteger32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER32* value )
{
	return readUnsigned32( nodeId, index, subindex, (SDO_t_UNSIGNED32*)value );
}

bool CCANopenAccess::readUnsigned8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED8* value )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_read_8bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_read_8bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::readUnsigned16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED16* value )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_read_16bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_read_16bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::readUnsigned32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED32* value )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = sdo_read_32bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_read_32bit( nodeId, index, subindex, value )) == COPERR_NOERROR)
		{
			m_lastAbort = 0x00000000L;
			return true;
		}
		else
		{
			if( m_lastError > COPERR_NOERROR)
			{
				m_lastAbort = m_lastError;
				m_lastError = COPERR_ABORTED;
			}
			return false;
		}
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::startRemoteNode( BYTE nodeId )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = nmt_start_remote_node( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_start_remote_node( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::stopRemoteNode( BYTE nodeId )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = nmt_stop_remote_node( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_stop_remote_node( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::enterPreOperationalState( BYTE nodeId )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = nmt_enter_preoperational( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_enter_preoperational( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::resetNode( BYTE nodeId )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = nmt_reset_node( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_reset_node( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::resetCommunication( BYTE nodeId )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = nmt_reset_communication( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_reset_communication( nodeId )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::transmitMessage( WORD COBId, SHORT length, BYTE* data )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = cop_transmit( COBId, length, data )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_transmit( COBId, length, data )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::requestMessage( WORD COBId, SHORT* length, BYTE* data )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = cop_request( COBId, length, data )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_request( COBId, length, data )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::readQueue( WORD* COBId, SHORT* length, BYTE* data )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = cop_queue_read( (LONG*)COBId, length, data )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_queue_read( (LONG*)COBId, length, data )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::clearQueue()
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( (m_lastError = cop_queue_clear()) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_queue_clear()) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::getFirmwareVersion( char* version, SHORT size )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( version )
		{
			strncpy( version, cop_software(), size );
			return true;
		}
		else
		{
			m_lastError = COPERR_NULLPTR;
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_software( version, size )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::getHardwareVersion( char* version, SHORT size )
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		if( version )
		{
			strncpy( version, cop_hardware(), size );
			return true;
		}
		else
		{
			m_lastError = COPERR_NULLPTR;
			return false;
		}
		break;
	case CCANopenAccess::libCANopenOverTcpIp:
		if( (m_lastError = top_hardware( version, size )) == COPERR_NOERROR)
			return true;
		else
			return false;
		break;
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

bool CCANopenAccess::getCopyright( char* copyright, SHORT size )
{
	if( copyright )
	{
		strncpy( copyright,_uv, size );
		return true;
	}
	else
	{
		m_lastError = COPERR_NULLPTR;
		return false;
	}
}

const char* CCANopenAccess::getAPIVersion()
{
	switch( m_library )
	{
	case CCANopenAccess::libSocketCAN:
		return (const char*)cop_version();
	case CCANopenAccess::libCANopenOverTcpIp:
		return (const char*)top_version();
	default:
		m_lastError = COPERR_NOTINIT;
		return false;
	}
}

const char* CCANopenAccess::getErrorText( int lastError )
{
	int i = 0;

	if( lastError == 0 )
		lastError = m_lastError;
	for( i = 0; error[i].code != COPERR_LAST_ERROR; i++ )
		if( error[i].code == lastError )
			break;
	return error[i].text;
}

const char* CCANopenAccess::getAbortText( long lastAbort )
{
	int i = 0;

	if( lastAbort == 0 )
		lastAbort = m_lastAbort;
	for( i = 0; abort[i].code != (long)0xFFFFFFFF; i++ )
		if( abort[i].code == lastAbort )
			break;
	return abort[i].text;
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CCANopenAccess::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
