////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/CANopenAccess.h $
//
//	project   :  CAN - Controller Area Network
//	
//	purpose   :  CANopen Master API (using SocketCAN)
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenAccess
//	
//	includes  :  CANopenTypes.h, cop/default.h
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Master API (using SocketCAN)

#ifndef _CANopenAccess_h_
#define _CANopenAccess_h_

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenTypes.h"
#include "cop/default.h"

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenAccess

class CCANopenAccess
{
	//	enumerations:
	public:
		enum {
			libUndefined = -1,
			libSocketCAN = 0,
			libCANopenOverTcpIp = 700
		};
	//	attributes:
	private:
		int  m_library;
		int  m_lastError;
		long m_lastAbort;
	
	//	constructor/destructor:
	public:
		CCANopenAccess();
		~CCANopenAccess();
		
	//	member functions:
	public:
		bool initialize( SHORT library, LONG board, LPVOID param, BYTE baudrate );
		bool terminate();

		bool reset( BYTE baudrate );

		bool write( BYTE nodeId, WORD index, BYTE subindex, SHORT length, BYTE* data );
		bool writeInteger8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER8 value );
		bool writeInteger16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER16 value );
		bool writeInteger32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER32 value );
		bool writeUnsigned8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED8 value );
		bool writeUnsigned16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED16 value );
		bool writeUnsigned32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED32 value );

		bool read( BYTE nodeId, WORD index, BYTE subindex, SHORT* length, BYTE* data, SHORT size );
		bool readInteger8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER8* value );
		bool readInteger16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER16* value );
		bool readInteger32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_INTEGER32* value );
		bool readUnsigned8( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED8* value );
		bool readUnsigned16( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED16* value );
		bool readUnsigned32( BYTE nodeId, WORD index, BYTE subindex, SDO_t_UNSIGNED32* value );

		bool startRemoteNode( BYTE nodeId );
		bool stopRemoteNode( BYTE nodeId );
		bool enterPreOperationalState( BYTE nodeId );
		bool resetNode( BYTE nodeId );
		bool resetCommunication( BYTE nodeId );

		bool transmitMessage( WORD COBId, SHORT length, BYTE* data );
		bool requestMessage( WORD COBId, SHORT* length, BYTE* data );

		bool readQueue( WORD* COBId, SHORT* length, BYTE* data );
		bool clearQueue();

		bool getFirmwareVersion( char* version, SHORT size );
		bool getHardwareVersion( char* version, SHORT size );
		bool getCopyright( char* copyright, SHORT size );

		bool setSDOTimeout( WORD newValue, WORD* oldValue = 0 );
		bool isSDOAborted() { return m_lastError == 4/*COPERR_ABORTED*/; };

		int  getLastError()	{ return m_lastError; }
		long getLastAbort()	{ return m_lastAbort; }

		const char* getErrorText( int lastError = 0 );
		const char* getAbortText( long lastAbort = 0 );

		const char* getAPIVersion();
	
	//	revision control:
	public:
		static const char* version();
};

#endif // _CANopenAccess_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
