////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/CANopenTypes.h $
//
//	project   :  CAN - Controller Area Network
//	
//	purpose   :  CANopen Data Types (Encoding rules)
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenTypes
//	
//	includes  :  <includes>
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	<description>

#ifndef _CANopenTypes_h_
#define _CANopenTypes_h_

//////////////////////////////////////////////////////////////////////////////
//	types

typedef signed char    SDO_t_INTEGER8;	// Type for signed 8-bit values
typedef signed short   SDO_t_INTEGER16;	// Type for signed 16-bit values
typedef signed long    SDO_t_INTEGER32;	// Type for signed 32-bit values

typedef unsigned char  SDO_t_UNSIGNED8;	// Type for unsigned 8-bit values
typedef unsigned short SDO_t_UNSIGNED16;// Type for unsigned 16-bit values
typedef unsigned long  SDO_t_UNSIGNED32;// Type for unsigned 32-bit values

typedef union _sdo_byte					// SDO 8-bit value:
{
	unsigned char  data[1];				//   byte buffer
	unsigned char  value;				//   8-bit value
}	SDO_t_BYTE;
typedef union _sdo_word					// SDO 16-bit value:
{
	unsigned char  data[2];				//   byte buffer
	unsigned short value;				//   16-bit value
}	SDO_t_WORD;
typedef union _sdo_dword				// SDO 32-bit value:
{
	unsigned char  data[4];				//   byte buffer
	unsigned long  value;				//   32-bit value
}	SDO_t_DWORD;
typedef union _sdo_abort				// SDO Abort-code:
{
	unsigned char  data[4];				//   byte buffer
	unsigned long  code;				//   32-bit value
}	SDO_t_ABORT;
typedef union _date						// Date (DS-301, V3.0):
{
	unsigned char  data[7];				//   byte buffer
	struct _date_member {				//   value:
		unsigned short milliseconds;	//     milliseconds (0,..,59999)
		unsigned char  minutes;			//     minutes (0,..,59)
		unsigned char  hour : 5;		//     hours (0,..,23)
		unsigned char  reserved : 2;	//     (not used)
		unsigned char  daylight_saving : 1;	// daylight saving time (0=No)
		unsigned char  day_of_month : 5;//     day of month (1,..,31)
		unsigned char  day_of_week : 3;	//     day of week (1=Monday)
		unsigned char  month;			//     month (1,..12)
		unsigned char  year;			//     year (0,..,99)
	}	__attribute__((packed)) member;
}	SDO_t_DATE;
typedef union _time_of_day {			// Date (time-of-day):
	unsigned char  data[6];				//   byte buffer
	struct _time_of_day_member {		//   value:
		unsigned long  msec;			//     milliseconds since midnight
		unsigned short days;			//     days since begin of the epoch
	}	__attribute__((packed)) member;
}	SDO_t_TIME_OF_DAY;

//////////////////////////////////////////////////////////////////////////////
//	forwarder

class QString;

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenTypes

class CCANopenTypes
{
	//	attributes:
	private:
	
	//	constructor/destructor:
	public:
		//CCANopenTypes();
		//~CCANopenTypes();
		
	//	member functions:
	public:
	
	//	static member
	public:
	static bool ascii2integer8( const char* string, SDO_t_INTEGER8* value );
	static bool ascii2integer16( const char* string, SDO_t_INTEGER16* value );
	static bool ascii2integer32( const char* string, SDO_t_INTEGER32* value );
	static bool ascii2unsigned8( const char* string, SDO_t_UNSIGNED8* value );
	static bool ascii2unsigned16( const char* string, SDO_t_UNSIGNED16* value );
	static bool ascii2unsigned32( const char* string, SDO_t_UNSIGNED32* value );
	static bool ascii2timeOfDay( const char* string, SDO_t_TIME_OF_DAY* value );
	static bool ascii2date( const char* string, SDO_t_DATE* value );

	static bool timeOfDay2ascii( SDO_t_TIME_OF_DAY timeOfDay, QString& ascii );
	static bool date2ascii( SDO_t_DATE date, QString& ascii );

	//	revision control:
	public:
		static const char* version();
};

#endif // _CANopenTypes_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
