////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/ConnectNodeDialog.cpp $
//
//	project   :  COPbrowser - CANopen Object Browser
//	
//	purpose   :  ConnectNodeDialog - Dialog
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CConnectNodeDialog
//	
//	includes  :  ConnectNodeDialog.h, <includes>
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	<description>

static const char _id[] = "$Id: ConnectNodeDialog.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "ConnectNodeDialog.h"
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>
#include "COPbrowser.h"

//////////////////////////////////////////////////////////////////////////////
//	class CConnectNodeDialog

CConnectNodeDialog::CConnectNodeDialog( QWidget* parent, const char * name )
	: QDialog( parent, name, FALSE, 0 )
{
	if ( !name )
		setName( "Connect Node dialog" );
	QVBoxLayout* dialogLayout = new QVBoxLayout( this, 11, 6, "Connect Node layout");

    m_nodeIdListBox = new QListBox( this, "Node-Id. list box" );
    m_nodeIdListBox->setRowMode( QListBox::FitToHeight );
    dialogLayout->addWidget( m_nodeIdListBox );

	QLabel* nodeIdLabel = new QLabel( "&Node-Id.:", this, "Node-Id. text label" );
	QLabel* fromToLabel = new QLabel( "(1,..,127)", this, "1,..,127 text label" );
    m_nodeIdSpinBox = new QSpinBox( this, "Node-Id. spin box" );
    m_nodeIdSpinBox->setMaxValue( 127 );
    m_nodeIdSpinBox->setMinValue( 1 );
    m_nodeIdSpinBox->setValue( 1 );
    nodeIdLabel->setBuddy( m_nodeIdSpinBox );

	m_connectPushButton = new QPushButton( this, "Connect push button" );
	m_connectPushButton->setText( "&Connect" );
	m_connectPushButton->setAccel( QKeySequence( "Alt+C" ) );
	m_connectPushButton->setDefault( TRUE );

	m_offlinePushButton = new QPushButton( this, "Offline push button" );
	m_offlinePushButton->setText( "&Offline" );
	m_offlinePushButton->setAccel( QKeySequence( "Alt+O" ) );

	m_searchPushButton = new QPushButton( this, "Search node push button" );
	m_searchPushButton->setText( "&Search node" );
	m_searchPushButton->setAccel( QKeySequence( "Alt+S" ) );

	QHBoxLayout* localLayout = new QHBoxLayout( 0, 0, 6, "local layout" );
	QGridLayout* localLeftLayout = new QGridLayout( 0, 1, 1, 0, 6, "local left layout" );

    localLeftLayout->addWidget( nodeIdLabel, 0, 0 );
    localLeftLayout->addWidget( m_nodeIdSpinBox, 0, 1 );
    localLeftLayout->addMultiCellWidget( fromToLabel, 0, 0, 2, 3 );
    localLeftLayout->addMultiCellWidget( m_searchPushButton, 1, 1, 1, 2 );
    QSpacerItem* spacer = new QSpacerItem( 31, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    localLeftLayout->addItem( spacer, 1, 3 );

    QVBoxLayout* localRightLayout = new QVBoxLayout( 0, 0, 6, "local right layout" );

    localRightLayout->addWidget( m_connectPushButton );
    localRightLayout->addWidget( m_offlinePushButton );

    localLayout->addLayout( localLeftLayout );
    localLayout->addLayout( localRightLayout );
	dialogLayout->addLayout( localLayout );

	// tool tip
	QToolTip::add( m_nodeIdListBox, CONNECT_NODE_NODE_LIST_STATUSTIP );
	QToolTip::add( m_nodeIdSpinBox, CONNECT_NODE_NODE_ID_STATUSTIP );
	QToolTip::add( m_connectPushButton, CONNECT_NODE_CONNECT_STATUSTIP );
	QToolTip::add( m_offlinePushButton, CONNECT_NODE_OFFLINE_STATUSTIP );
	QToolTip::add( m_searchPushButton, CONNECT_NODE_SEARCH_STATUSTIP );

	// context help
	QWhatsThis::add( nodeIdLabel, CONNECT_NODE_NODE_ID_STATUSTIP );
	QWhatsThis::add( fromToLabel, CONNECT_NODE_NODE_ID_STATUSTIP );
	QWhatsThis::add( m_nodeIdListBox, CONNECT_NODE_NODE_LIST_WHATsTHIS );
	QWhatsThis::add( m_nodeIdSpinBox, CONNECT_NODE_NODE_ID_WHATsTHIS );
	QWhatsThis::add( m_connectPushButton, CONNECT_NODE_CONNECT_WHATsTHIS );
	QWhatsThis::add( m_offlinePushButton, CONNECT_NODE_OFFLINE_WHATsTHIS );
	QWhatsThis::add( m_searchPushButton, CONNECT_NODE_SEARCH_WHATsTHIS );

    // signal/slot connections
	connect( m_nodeIdListBox, SIGNAL( doubleClicked( QListBoxItem * ) ),
			              this, SLOT( doubleClicked( QListBoxItem * ) ) );
	connect( m_connectPushButton, SIGNAL( clicked() ), this, SLOT( acceptConnect() ) );
    connect( m_offlinePushButton, SIGNAL( clicked() ), this, SLOT( acceptOffline() ) );
    connect( m_searchPushButton, SIGNAL( clicked() ), this, SLOT( searchNode() ) );

    // populate the list box
    for( int i = 1; i <= 127; i++ )
    {
    	m_nodeIdListBox->insertItem( QString( "Node %1" ).arg( i ) );
    }
    m_connectFlag = false;

	// resize the dialog box
    resize( QSize(500, 310).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );
    setCaption( "Connect a Node" );
}

CConnectNodeDialog::~CConnectNodeDialog()
{
}

int CConnectNodeDialog::getNodeId()
{
	return m_nodeIdSpinBox->value();
}

bool CConnectNodeDialog::getConnectFlag()
{
	return m_connectFlag;
}

void CConnectNodeDialog::doubleClicked( QListBoxItem * item )
{
	if( item && (m_nodeIdListBox->index( item ) != -1) )
	{
		m_nodeIdSpinBox->setValue( m_nodeIdListBox->index( item ) + 1 );
		// connect the node
		acceptConnect();
	}
}

void CConnectNodeDialog::acceptConnect()
{
	if( (1 <= m_nodeIdSpinBox->value()) && (m_nodeIdSpinBox->value() <= 127) )
	{
		// set connect flag
	    m_connectFlag = true;
		done( QDialog::Accepted );
	}
	else
	{
		QString err = QString( "Invalid node-id. given!\n\nPlease retry." );
		QMessageBox::warning( this, PROGRAM_STRING" - Connect a Node", err);
	}
}

void CConnectNodeDialog::acceptOffline()
{
	// clear connect flag
    m_connectFlag = false;
	done( QDialog::Accepted );
}

void CConnectNodeDialog::searchNode()
{
	//@ToDo: search dialog!
	//...
	QMessageBox::critical( this, PROGRAM_STRING" - Search node", "Sorry, not realized yet!" );
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CConnectNodeDialog::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
