////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/MDIWindow.cpp $
//
//	project   :  COPbrowser - CANopen Object Browser
//
//	purpose   :  MDIWindow - MDI (Multi Document Interface) Window
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CMDIWindow
//
//	includes  :  MDIWindow.h, COPbrowser.h, DeviceInfoDialog.h,
//	             CANopenTypes.h, dic/CANopenReader.h,
//	             cop/cop_api.h, cop/default.h
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	MDIWindow - MDI (Multi Document Interface) Window

static const char _id[] = "$Id: MDIWindow.cpp 60 2009-11-07 21:47:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "COPbrowser.h"
#include "MDIWindow.h"
#include <qstring.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qvbox.h>
#include <qlistview.h>
#include <qheader.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <qsettings.h>
#include <qmessagebox.h>
#include <qapplication.h>

#include "ConnectNodeDialog.h"
#include "DeviceInfoDialog.h"
#include "dic/CANopenReader.h"
#include "CANopenAccess.h"

//////////////////////////////////////////////////////////////////////////////
//	class CMDIWindow

CMDIWindow::CMDIWindow( QWidget* parent, const char* name, WFlags wflags )
	: QMainWindow( parent, name, wflags )
{
	// attribute initialization
	m_CANopen = NULL;
	m_nodeId = 0;
	m_index = 0x0000;
	m_subIndex = 0xFF;

	// client window: multi-document interface
	setCentralWidget( new QWidget( this, "client central widget" ) );
	QVBoxLayout* MDIWindowLayout = new QVBoxLayout( centralWidget(), 11, 6, "client window layout");

	// splitter window for the object dictionary
	QSplitter* splitter = new QSplitter( centralWidget(), "client splitter window"  );
	splitter->setOrientation( QSplitter::Horizontal );

	// splitter left side: list view for the object list
	m_dictionaryListView = new QListView( splitter, "object dictionary list view" );
	m_dictionaryListView->addColumn( "CANopen: Object dictionary       " );
	m_dictionaryListView->setRootIsDecorated( true );
	m_dictionaryListView->setSelectionMode( QListView::Single );
	m_dictionaryListView->setColumnWidthMode( 0, QListView::Manual );
	m_dictionaryListView->header()->hide();

	// splitter right side: form view for the object attributes
	QWidget* attributesFormWidget = new QWidget( splitter, "object attributes form widget" );

	// - group box: object description
	m_objectGroupBox = new QGroupBox( attributesFormWidget, "object description group box" );
	m_objectGroupBox->setTitle( "Object description" );
	m_objectGroupBox->setColumnLayout(0, Qt::Vertical );
	m_objectGroupBox->layout()->setSpacing( 6 );
	m_objectGroupBox->layout()->setMargin( 11 );

	QLabel* objectNameLabel = new QLabel( "Object name:", m_objectGroupBox, "object name text label" );
	m_objectDescription.m_nameLineEdit = new QLineEdit( m_objectGroupBox, "object name line edit" );
	m_objectDescription.m_nameLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_objectDescription.m_nameLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_objectDescription.m_nameLineEdit->setReadOnly( TRUE );;
	objectNameLabel->setBuddy( m_objectDescription.m_nameLineEdit );

	QLabel* objectCodeLabel = new QLabel( "Object code:", m_objectGroupBox, "object code text label" );
	m_objectDescription.m_codeLineEdit = new QLineEdit( m_objectGroupBox, "object code line edit" );
	m_objectDescription.m_codeLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_objectDescription.m_codeLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_objectDescription.m_codeLineEdit->setReadOnly( TRUE );;
	objectCodeLabel->setBuddy( m_objectDescription.m_codeLineEdit );

	QLabel* objectTypeLabel = new QLabel( "Data type:", m_objectGroupBox, "object data type text label" );
	m_objectDescription.m_typeLineEdit = new QLineEdit( m_objectGroupBox, "object data type line edit" );
	m_objectDescription.m_typeLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_objectDescription.m_typeLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_objectDescription.m_typeLineEdit->setReadOnly( TRUE );;
	objectTypeLabel->setBuddy( m_objectDescription.m_typeLineEdit );

	m_objectDescription.m_compoundCheckBox = new QCheckBox( "Compound object", m_objectGroupBox, "compound object check box" );
	m_objectDescription.m_mandatoryCheckBox = new QCheckBox( "Mandatory object", m_objectGroupBox, "mandatory object check box" );

	QGridLayout* objectGroupBoxLayout = new QGridLayout( m_objectGroupBox->layout() );
	objectGroupBoxLayout->setAlignment( Qt::AlignTop );;
	objectGroupBoxLayout->addWidget( objectNameLabel, 0, 0 );
	objectGroupBoxLayout->addWidget( objectCodeLabel, 1, 0 );
	objectGroupBoxLayout->addWidget( objectTypeLabel, 2, 0 );
	objectGroupBoxLayout->addWidget( m_objectDescription.m_nameLineEdit, 0, 1 );
	objectGroupBoxLayout->addWidget( m_objectDescription.m_codeLineEdit, 1, 1 );
	objectGroupBoxLayout->addWidget( m_objectDescription.m_typeLineEdit, 2, 1 );
	objectGroupBoxLayout->addWidget( m_objectDescription.m_compoundCheckBox, 1, 2 );
	objectGroupBoxLayout->addWidget( m_objectDescription.m_mandatoryCheckBox, 2, 2 );

	// - group box: entry description
	m_entryGroupBox = new QGroupBox( attributesFormWidget, "entry description group box" );
	m_entryGroupBox->setTitle( "Entry description" );
	m_entryGroupBox->setColumnLayout(0, Qt::Vertical );
	m_entryGroupBox->layout()->setSpacing( 6 );
	m_entryGroupBox->layout()->setMargin( 11 );

	QLabel* entryNameLabel = new QLabel( "Parameter name:", m_entryGroupBox, "parameter name text label" );
	m_entryDescription.m_nameLineEdit = new QLineEdit( m_entryGroupBox, "parameter name line edit" );
	m_entryDescription.m_nameLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_entryDescription.m_nameLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_entryDescription.m_nameLineEdit->setReadOnly( TRUE );;
	entryNameLabel->setBuddy( m_entryDescription.m_nameLineEdit );

	QLabel* entryTypeLabel = new QLabel( "Data type:", m_entryGroupBox, "parameter data type text label" );
	m_entryDescription.m_typeLineEdit = new QLineEdit( m_entryGroupBox, "parameter data type line edit" );
	m_entryDescription.m_typeLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_entryDescription.m_typeLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_entryDescription.m_typeLineEdit->setReadOnly( TRUE );;
	entryTypeLabel->setBuddy( m_entryDescription.m_typeLineEdit );

	QLabel* entryAccessLabel = new QLabel( "Access type:", m_entryGroupBox, "access type text label" );
	m_entryDescription.m_accessLineEdit = new QLineEdit( m_entryGroupBox, "access type line edit" );
	m_entryDescription.m_accessLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_entryDescription.m_accessLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_entryDescription.m_accessLineEdit->setReadOnly( TRUE );;
	entryAccessLabel->setBuddy( m_entryDescription.m_accessLineEdit );

	QLabel* entryRangeLabel = new QLabel( "Value range:", m_entryGroupBox, "value range text label" );
	m_entryDescription.m_rangeLineEdit = new QLineEdit( m_entryGroupBox, "value range line edit" );
	m_entryDescription.m_rangeLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_entryDescription.m_rangeLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_entryDescription.m_rangeLineEdit->setReadOnly( TRUE );;
	entryRangeLabel->setBuddy( m_entryDescription.m_rangeLineEdit );

	QLabel* entryDefaultLabel = new QLabel( "Default value:", m_entryGroupBox, "default value text label" );
	m_entryDescription.m_defaultLineEdit = new QLineEdit( m_entryGroupBox, "default value line edit" );
	m_entryDescription.m_defaultLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_entryDescription.m_defaultLineEdit->setFrameShape( QLineEdit::NoFrame );
	m_entryDescription.m_defaultLineEdit->setReadOnly( TRUE );;
	entryDefaultLabel->setBuddy( m_entryDescription.m_defaultLineEdit );

	m_entryDescription.m_mandatoryCheckBox = new QCheckBox( "Mandatory entry", m_entryGroupBox, "mandatory entry check box" );
	m_entryDescription.m_mappingCheckBox = new QCheckBox( "PDO Mapping", m_entryGroupBox, "mapped entry check box" );

	QGridLayout* entryGroupBoxLayout = new QGridLayout( m_entryGroupBox->layout() );
	entryGroupBoxLayout->setAlignment( Qt::AlignTop );;
	entryGroupBoxLayout->addWidget( entryNameLabel, 0, 0 );
	entryGroupBoxLayout->addWidget( entryTypeLabel, 1, 0 );
	entryGroupBoxLayout->addWidget( entryAccessLabel, 2, 0 );
	entryGroupBoxLayout->addWidget( entryRangeLabel, 3, 0 );
	entryGroupBoxLayout->addWidget( entryDefaultLabel, 4, 0 );
	entryGroupBoxLayout->addWidget( m_entryDescription.m_nameLineEdit, 0, 1 );
	entryGroupBoxLayout->addWidget( m_entryDescription.m_typeLineEdit, 1, 1 );
	entryGroupBoxLayout->addWidget( m_entryDescription.m_accessLineEdit, 2, 1 );
	entryGroupBoxLayout->addWidget( m_entryDescription.m_rangeLineEdit, 3, 1 );
	entryGroupBoxLayout->addWidget( m_entryDescription.m_defaultLineEdit, 4, 1 );
	entryGroupBoxLayout->addWidget( m_entryDescription.m_mandatoryCheckBox, 1, 2 );
	entryGroupBoxLayout->addWidget( m_entryDescription.m_mappingCheckBox, 2, 2 );

	// - group box: data access
	m_accessGroupBox = new QGroupBox( attributesFormWidget, "data access group box" );
	m_accessGroupBox->setTitle( "Data access" );
	m_accessGroupBox->setColumnLayout(0, Qt::Vertical );
	m_accessGroupBox->layout()->setSpacing( 6 );
	m_accessGroupBox->layout()->setMargin( 11 );

	QLabel* accessInputLabel = new QLabel( "Enter new &value:", m_accessGroupBox, "new value text label" );
	m_dataAccess.m_inputLineEdit = new QLineEdit( m_accessGroupBox, "new value line edit" );
	accessInputLabel->setBuddy( m_dataAccess.m_inputLineEdit );

	QLabel* accessOutputLabel = new QLabel( "Parameter value:", m_accessGroupBox, "actual value text label" );
	m_dataAccess.m_outputLineEdit = new QLineEdit( m_accessGroupBox, "actual value line edit" );
	m_dataAccess.m_outputLineEdit->setPaletteBackgroundColor( centralWidget()->paletteBackgroundColor() );
	m_dataAccess.m_outputLineEdit->setReadOnly( TRUE );
	accessOutputLabel->setBuddy( m_dataAccess.m_outputLineEdit );

	m_dataAccess.m_sendPushButton = new QPushButton( "&Send", m_accessGroupBox, "send push button" );
	m_dataAccess.m_readPushButton = new QPushButton( "&Read", m_accessGroupBox, "read push button" );

	m_dataAccess.m_hexRadioButton = new QRadioButton( "&hexadecimal", m_accessGroupBox, "hex radio button" );
	m_dataAccess.m_decRadioButton = new QRadioButton( "&decimal", m_accessGroupBox, "dec radio button" );
	m_dataAccess.m_octRadioButton = new QRadioButton( "&octal", m_accessGroupBox, "oct radio button" );

	QGridLayout* accessGroupBoxLayout = new QGridLayout( m_accessGroupBox->layout() );
	accessGroupBoxLayout->setAlignment( Qt::AlignTop );;
	accessGroupBoxLayout->addWidget( accessInputLabel, 0, 0 );
	accessGroupBoxLayout->addWidget( accessOutputLabel, 1, 0 );
	accessGroupBoxLayout->addWidget( m_dataAccess.m_inputLineEdit, 0, 1 );
	accessGroupBoxLayout->addWidget( m_dataAccess.m_outputLineEdit, 1, 1 );
	accessGroupBoxLayout->addWidget( m_dataAccess.m_sendPushButton, 0, 2 );
	accessGroupBoxLayout->addWidget( m_dataAccess.m_readPushButton, 1, 2 );
	   QHBoxLayout* displayModeLayout = new QHBoxLayout( 0, 0, 6, "display mode buttons layout");
	   displayModeLayout->addWidget( m_dataAccess.m_hexRadioButton );
	   displayModeLayout->addWidget( m_dataAccess.m_decRadioButton );
	   displayModeLayout->addWidget( m_dataAccess.m_octRadioButton );
	accessGroupBoxLayout->addLayout( displayModeLayout, 2, 1 );

	// layout of the client windows and its widgets
	QVBoxLayout* attributesFormLayout = new QVBoxLayout( attributesFormWidget, 11, 6 );
	attributesFormLayout->addWidget( m_objectGroupBox );
	attributesFormLayout->addWidget( m_entryGroupBox );
	attributesFormLayout->addWidget( m_accessGroupBox );
	QSpacerItem* spacer = new QSpacerItem( 6, 11, QSizePolicy::Minimum, QSizePolicy::Expanding );
	attributesFormLayout->addItem( spacer );
	MDIWindowLayout->addWidget( splitter );

	// tool tip
	QToolTip::add( m_dictionaryListView, OBJECT_DICTIONARY_STATUSTIP );
	QToolTip::add( m_objectDescription.m_nameLineEdit, OBJECT_NAME_STATUSTIP );
	QToolTip::add( m_objectDescription.m_codeLineEdit, OBJECT_CODE_STATUSTIP );
	QToolTip::add( m_objectDescription.m_typeLineEdit, OBJECT_TYPE_STATUSTIP );
	QToolTip::add( m_objectDescription.m_compoundCheckBox, OBJECT_COMPOUND_STATUSTIP );
	QToolTip::add( m_objectDescription.m_mandatoryCheckBox, OBJECT_MANDATORY_STATUSTIP );
	QToolTip::add( m_entryDescription.m_nameLineEdit, ENTRY_NAME_STATUSTIP );
	QToolTip::add( m_entryDescription.m_typeLineEdit, ENTRY_TYPE_STATUSTIP );
	QToolTip::add( m_entryDescription.m_accessLineEdit, ENTRY_ACCESS_STATUSTIP );
	QToolTip::add( m_entryDescription.m_rangeLineEdit, ENTRY_RANGE_STATUSTIP );
	QToolTip::add( m_entryDescription.m_defaultLineEdit, ENTRY_DEFAULT_STATUSTIP );
	QToolTip::add( m_entryDescription.m_mandatoryCheckBox, ENTRY_MANDATORY_STATUSTIP );
	QToolTip::add( m_entryDescription.m_mappingCheckBox, ENTRY_MAPPING_STATUSTIP );
	QToolTip::add( m_dataAccess.m_inputLineEdit, ACCESS_INPUT_STATUSTIP );
	QToolTip::add( m_dataAccess.m_outputLineEdit, ACCESS_OUTPUT_STATUSTIP );
	QToolTip::add( m_dataAccess.m_hexRadioButton, ACCESS_HEX_STATUSTIP );
	QToolTip::add( m_dataAccess.m_decRadioButton, ACCESS_DEC_STATUSTIP );
	QToolTip::add( m_dataAccess.m_octRadioButton, ACCESS_OCT_STATUSTIP );
	QToolTip::add( m_dataAccess.m_sendPushButton, ACCESS_SEND_STATUSTIP );
	QToolTip::add( m_dataAccess.m_readPushButton, ACCESS_READ_STATUSTIP );

	// context help
	QWhatsThis::add( centralWidget(), MDI_WINDOW_WHATsTHIS );
	QWhatsThis::add( objectNameLabel, OBJECT_NAME_WHATsTHIS );
	QWhatsThis::add( objectCodeLabel, OBJECT_CODE_WHATsTHIS );
	QWhatsThis::add( objectTypeLabel, OBJECT_TYPE_WHATsTHIS );
	QWhatsThis::add( entryNameLabel, ENTRY_NAME_WHATsTHIS );
	QWhatsThis::add( entryTypeLabel, ENTRY_TYPE_WHATsTHIS );
	QWhatsThis::add( entryAccessLabel, ENTRY_ACCESS_WHATsTHIS );
	QWhatsThis::add( entryRangeLabel, ENTRY_RANGE_WHATsTHIS );
	QWhatsThis::add( entryDefaultLabel, ENTRY_DEFAULT_WHATsTHIS );
	QWhatsThis::add( accessInputLabel, ACCESS_INPUT_WHATsTHIS );
	QWhatsThis::add( accessOutputLabel, ACCESS_OUTPUT_WHATsTHIS );
	QWhatsThis::add( m_objectDescription.m_nameLineEdit, OBJECT_NAME_WHATsTHIS );
	QWhatsThis::add( m_objectDescription.m_codeLineEdit, OBJECT_CODE_WHATsTHIS );
	QWhatsThis::add( m_objectDescription.m_typeLineEdit, OBJECT_TYPE_WHATsTHIS );
	QWhatsThis::add( m_objectDescription.m_compoundCheckBox, OBJECT_COMPOUND_WHATsTHIS );
	QWhatsThis::add( m_objectDescription.m_mandatoryCheckBox, OBJECT_MANDATORY_WHATsTHIS );
	QWhatsThis::add( m_entryDescription.m_nameLineEdit, ENTRY_NAME_WHATsTHIS );
	QWhatsThis::add( m_entryDescription.m_typeLineEdit, ENTRY_TYPE_WHATsTHIS );
	QWhatsThis::add( m_entryDescription.m_accessLineEdit, ENTRY_ACCESS_WHATsTHIS );
	QWhatsThis::add( m_entryDescription.m_rangeLineEdit, ENTRY_RANGE_WHATsTHIS );
	QWhatsThis::add( m_entryDescription.m_defaultLineEdit, ENTRY_DEFAULT_WHATsTHIS );
	QWhatsThis::add( m_entryDescription.m_mandatoryCheckBox, ENTRY_MANDATORY_WHATsTHIS );
	QWhatsThis::add( m_entryDescription.m_mappingCheckBox, ENTRY_MAPPING_WHATsTHIS );
	QWhatsThis::add( m_dataAccess.m_inputLineEdit, ACCESS_INPUT_WHATsTHIS );
	QWhatsThis::add( m_dataAccess.m_outputLineEdit, ACCESS_OUTPUT_WHATsTHIS );
	QWhatsThis::add( m_dataAccess.m_hexRadioButton, ACCESS_HEX_WHATsTHIS );
	QWhatsThis::add( m_dataAccess.m_decRadioButton, ACCESS_DEC_WHATsTHIS );
	QWhatsThis::add( m_dataAccess.m_octRadioButton, ACCESS_OCT_WHATsTHIS );
	QWhatsThis::add( m_dataAccess.m_sendPushButton, ACCESS_SEND_WHATsTHIS );
	QWhatsThis::add( m_dataAccess.m_readPushButton, ACCESS_READ_WHATsTHIS );

	// signal/slot connections
	connect( m_dictionaryListView, SIGNAL( pressed( QListViewItem* ) ), this, SLOT( mouseLeftPressed( QListViewItem* ) ) );
	connect( m_dictionaryListView, SIGNAL( returnPressed( QListViewItem* ) ), this, SLOT( returnKeyPressed( QListViewItem* ) ) );
	connect( m_dataAccess.m_inputLineEdit, SIGNAL( returnPressed() ), this, SLOT( buttonSendClicked() ) );
	connect( m_dataAccess.m_sendPushButton, SIGNAL( clicked() ), this, SLOT( buttonSendClicked() ) );
	connect( m_dataAccess.m_readPushButton, SIGNAL( clicked() ), this, SLOT( buttonReadClicked() ) );
	connect( m_dataAccess.m_hexRadioButton, SIGNAL( clicked() ), this, SLOT( buttonHexClicked() ) );
	connect( m_dataAccess.m_decRadioButton, SIGNAL( clicked() ), this, SLOT( buttonDecClicked() ) );
	connect( m_dataAccess.m_octRadioButton, SIGNAL( clicked() ), this, SLOT( buttonOctClicked() ) );

	// last settings
	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	m_dataAccess.m_displayMode = (CMDIWindow::TDisplayMode)settings.readNumEntry( QString( "/%1/%2/%3" )
			.arg( APPLICATION_KEY )
			.arg( SETTINGS_KEY )
			.arg( DISPLAY_MODE_KEY ) );
	m_dataAccess.m_hexRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeHex );
	m_dataAccess.m_decRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeDec );
	m_dataAccess.m_octRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeOct );

	// resize to fit
	resize( QSize(720, 400).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );
}

CMDIWindow::~CMDIWindow()
{
}

void CMDIWindow::closeEvent( QCloseEvent* event )
{
	//@ToDo: save modified?
	event->accept();
}

bool CMDIWindow::openDocument( const QString& fileName, CCANopenAccess* CANopen )
{
	//@ToDo: determine document type
	CCANopen::TFileType fileType = CCANopen::typeEDS;
	//...
	// load document based on type
	CCANopenReader reader( &m_dictionary, fileType );
	if( !reader.load( fileName ) )
	{
		QString msg = QString( "File %1 could not be loaded" ).arg( fileName );
		emit( message( msg, 2000 ) );
		QMessageBox::critical( this, PROGRAM_STRING" - Load document", msg );
		return false;
	}
	// connect a node
	if( CANopen )
	{
		CConnectNodeDialog* dialog = new CConnectNodeDialog( this );
		if( dialog->exec() != QDialog::Accepted )
		{
			return false;
		}
		// get entered information
		if( dialog->getConnectFlag() )
			m_nodeId = (unsigned char)dialog->getNodeId();
		else
			m_nodeId = (unsigned char)0;
		m_CANopen = CANopen;
	}
	// populate the view
	populateDictionaryList();
	// set document name
	m_fileName = fileName;
	setCaption( m_fileName );

	if( (1 <= m_nodeId) && (m_nodeId <= 127) )
		emit( message( QString( "File %1 loaded and connected to node #%2" ).arg( m_fileName ).arg( m_nodeId ), 2000 ) );
	else
		emit( message( QString( "File %1 loaded in off-line mode" ).arg( m_fileName ), 2000 ) );

	return true;
}

void CMDIWindow::showDeviceInfo()
{
	CDeviceInfoDialog* dialog = new CDeviceInfoDialog( this );
	if( !dialog->populate( &m_dictionary ) )
	{
		emit( message( "No device info present", 2000 ) );
	}
	dialog->exec();
}

void CMDIWindow::populateDictionaryList()
{
	QListViewItem* itemRoot = NULL;
	QListViewItem* itemDS301 = NULL;
	QListViewItem* itemOther = NULL;
	QListViewItem* itemObject = NULL;
	QString rootText;

	if( (1 <= m_nodeId) && (m_nodeId <= 127) )
		rootText = QString( "CANopen: Node #%1" ).arg( m_nodeId );
	else
		rootText = QString( "CANopen: Object dictionary" );

	itemRoot = new QListViewItem( m_dictionaryListView, rootText );

	for( unsigned i = 0; i < m_dictionary.m_objectArray.count(); i++ )
	{
		CCANopenObject* ptrObject = m_dictionary.m_objectArray[i];
		ASSERT( ptrObject );
		QString strObject = QString( "%1: %2" )
							   .arg( ptrObject->getIndex(), 0, 16 )
							   .arg( ptrObject->getName() );

		if( ptrObject->getProfileNumber().compare( "CiA DS-301" ) == 0 )
		{
			if( itemDS301 == NULL )
			{
				itemDS301 = new QListViewItem( itemRoot, "CiA DS-301" );
			}
			itemObject = new QListViewItem( itemDS301, strObject );
		}
		else if( !ptrObject->getProfileNumber().isEmpty() )
		{
			QListViewItem* itemProfile = itemRoot->firstChild();
			while( itemProfile != NULL )
			{
				if( itemProfile->text( 0 ).compare( ptrObject->getProfileNumber() ) == 0 )
					break;
				itemProfile = itemProfile->nextSibling();
			}
			if( itemProfile == NULL )
			{
				itemProfile = new QListViewItem( itemRoot, ptrObject->getProfileNumber() );
				itemObject = new QListViewItem( itemProfile, strObject );
			}
			else if( !ptrObject->getProfileChapter().isEmpty() )
			{
				QListViewItem* itemChapter = itemProfile->firstChild();
				while( itemChapter != NULL )
				{
					if( itemChapter->text( 0 ).compare( ptrObject->getProfileChapter() ) == 0 )
						break;
					itemChapter = itemChapter->nextSibling();
				}
				if( itemChapter == NULL )
				{
					itemChapter = new QListViewItem( itemRoot, ptrObject->getProfileChapter() );
				}
				itemObject = new QListViewItem( itemChapter, strObject );
			}
			else
			{
				itemObject = new QListViewItem( itemProfile, strObject );
			}
		}
		else
		{
			if( itemOther == NULL )
			{
				itemOther = new QListViewItem( itemRoot, "Other objects" );
			}
			itemObject = new QListViewItem( itemOther, strObject );
		}
		if( (ptrObject->getClass() == CCANopenObject::classARRAY) ||
			(ptrObject->getClass() == CCANopenObject::classRECORD) ||
			(ptrObject->m_objectEntryArray.count() > 1) )
		{
			for( unsigned j = 0; j < ptrObject->m_objectEntryArray.count(); j++ )
			{
				CCANopenObjectEntry* entry = ptrObject->m_objectEntryArray[j];
				ASSERT( entry );

				new QListViewItem( itemObject, QString( "%1: %2" )
												  .arg( entry->getSubindex(), 2, 16)
												  .arg( entry->getName() ) );
			}
		}

	}
	if( itemDS301 != NULL )
		itemDS301->setOpen( true );
	itemRoot->setOpen( true );
}

void CMDIWindow::populateAttributesForm( unsigned short index, unsigned char subIndex )
{
	QString msg;

	if( m_index != index )
	{
		clearAttributesForm();

		if( index != 0x0000 )
		{
			int o = m_dictionary.find( index );

			if( (0 <= o ) && (o < (int)m_dictionary.m_objectArray.count()) )
			{
				CCANopenObject* ptrObject = m_dictionary.m_objectArray[o];
				ASSERT( ptrObject );
				m_objectGroupBox->setTitle( QString( "Object: Index %1h" ).arg( ptrObject->getIndex(), 0, 16 ) );
				m_objectDescription.m_nameLineEdit->setText( ptrObject->getName() );
				m_objectDescription.m_codeLineEdit->setText( CCANopenObject::getClassByIndex( ptrObject->getClass() ) );
				m_objectDescription.m_typeLineEdit->setText( CCANopenObject::getDataTypeByIndex( ptrObject->getDataType() ) );
				m_objectDescription.m_compoundCheckBox->setChecked( ptrObject->isMultipleObject() );
				m_objectDescription.m_mandatoryCheckBox->setChecked( ptrObject->getCategory() != CCANopenObject::categoryOptional);

				int e = ptrObject->find( subIndex );

				if( (0 <= e ) && (e < (int)ptrObject->m_objectEntryArray.count()) )
				{
					CCANopenObjectEntry* ptrEntry = ptrObject->m_objectEntryArray[e];
					ASSERT( ptrEntry );
					m_entryGroupBox->setTitle( QString( "Entry: Subndex %1h" ).arg( ptrEntry->getSubindex(), 2, 16) );
					m_entryDescription.m_nameLineEdit->setText( ptrEntry->getName() );
					m_entryDescription.m_typeLineEdit->setText( CCANopenObjectEntry::getDataTypeByIndex( ptrEntry->getDataType() ) );
					m_entryDescription.m_accessLineEdit->setText( CCANopenObjectEntry::getAccessByIndex( ptrEntry->getAccess() ) );
					m_entryDescription.m_rangeLineEdit->setText( ptrEntry->getValueRange() );
					m_entryDescription.m_defaultLineEdit->setText( ptrEntry->getDefaultValue() );
					m_entryDescription.m_mandatoryCheckBox->setChecked( ptrEntry->getCategory() != CCANopenObjectEntry::categoryOptional );
					m_entryDescription.m_mappingCheckBox->setChecked( ptrEntry->getPDOMapping() != CCANopenObjectEntry::mappingNo );
				}
				else
				{
					subIndex = 0xFF;
				}
			}
			else
			{
				index = 0x0000;
			}
		}
		m_subIndex = subIndex;
		m_index = index;
	}
	else if( m_subIndex != subIndex )
	{
		clearAttributesForm( true );

		if( subIndex != 0xFF )
		{
			int o = m_dictionary.find( index );

			if( (0 <= o ) && (o < (int)m_dictionary.m_objectArray.count()) )
			{
				CCANopenObject* ptrObject = m_dictionary.m_objectArray[o];
				ASSERT( ptrObject );

				int e = ptrObject->find( subIndex );

				if( (0 <= e ) && (e < (int)ptrObject->m_objectEntryArray.count()) )
				{
					CCANopenObjectEntry* ptrEntry = ptrObject->m_objectEntryArray[e];
					ASSERT( ptrEntry );
					m_entryGroupBox->setTitle( QString( "Entry: Subndex %1h" ).arg( ptrEntry->getSubindex(), 2, 16) );
					m_entryDescription.m_nameLineEdit->setText( ptrEntry->getName() );
					m_entryDescription.m_typeLineEdit->setText( CCANopenObjectEntry::getDataTypeByIndex( ptrEntry->getDataType() ) );
					m_entryDescription.m_accessLineEdit->setText( CCANopenObjectEntry::getAccessByIndex( ptrEntry->getAccess() ) );
					m_entryDescription.m_rangeLineEdit->setText( ptrEntry->getValueRange() );
					m_entryDescription.m_defaultLineEdit->setText( ptrEntry->getDefaultValue() );
					m_entryDescription.m_mandatoryCheckBox->setChecked( ptrEntry->getCategory() != CCANopenObjectEntry::categoryOptional );
					m_entryDescription.m_mappingCheckBox->setChecked( ptrEntry->getPDOMapping() != CCANopenObjectEntry::mappingNo );
				}
				else
				{
					subIndex = 0xFF;
					index = 0x0000;
				}
			}
			else
			{
				subIndex = 0xFF;
				index = 0x0000;
			}
		}
		m_subIndex = subIndex;
	}
	if( (1 <= m_nodeId) && (m_nodeId <= 127) )
	{
		m_accessGroupBox->setTitle( QString( "Data access: #%1").arg( m_nodeId ) );

		if( (m_index != 0x0000) && (m_subIndex != 0xFF) )
		{
			buttonReadClicked();
		}
	}
	else
	{
		m_accessGroupBox->setTitle( "Data access: N/A" );
	}
	if( index != 0x000 )
	{
		if( subIndex != 0xFF )
			msg = QString( "Attributes for object %1.%2 loaded" )
						   .arg( index, 0, 16 )
						   .arg( subIndex, 0, 16 );
		else
			msg = QString( "Attributes for object %1 loaded" )
						   .arg( index, 0, 16 );
	}
	else
	{
		msg = "No object attributes loaded";
	}
	if( index == 0x000 )//?
		emit( message( msg, 2000 ) );
}

void CMDIWindow::clearAttributesForm( bool entryOnly )
{
	if( !entryOnly )
	{
		m_objectGroupBox->setTitle( "Object description" );
		m_objectDescription.m_nameLineEdit->setText( "" );
		m_objectDescription.m_codeLineEdit->setText( "" );
		m_objectDescription.m_typeLineEdit->setText( "" );
		m_objectDescription.m_compoundCheckBox->setChecked( false );
		m_objectDescription.m_mandatoryCheckBox->setChecked( false );
		m_index = 0x0000;
	}
	m_entryGroupBox->setTitle( "Entry description" );
	m_entryDescription.m_nameLineEdit->setText( "" );
	m_entryDescription.m_typeLineEdit->setText( "" );
	m_entryDescription.m_accessLineEdit->setText( "" );
	m_entryDescription.m_rangeLineEdit->setText( "" );
	m_entryDescription.m_defaultLineEdit->setText( "" );
	m_entryDescription.m_mandatoryCheckBox->setChecked( false );
	m_entryDescription.m_mappingCheckBox->setChecked( false );
	m_accessGroupBox->setTitle( "Data access" );
	m_dataAccess.m_inputLineEdit->setText( "" );
	m_dataAccess.m_outputLineEdit->setText( "" );
	m_subIndex = 0xFF;
}

void CMDIWindow::mouseLeftPressed( QListViewItem* item )
{
	if( item )
	{
		unsigned short index = 0x0000;
		unsigned char subIndex = 0xFF;

		if( item->text( 0 ) )
		{
			char string[256];
			int num1 = -1, num2 = -1;

			if( sscanf( item->text( 0 ).ascii(), "%x: %s", &num1, string) == 2 )
			{
				QListViewItem* parent = item->parent();

				if( parent && parent->text( 0 ) )
				{
					if( sscanf( parent->text( 0 ).ascii(), "%x: %s", &num2, string) == 2 )
					{
						index = (unsigned short)num2;
						subIndex = (unsigned char)num1;
					}
					else
					{
						index = (unsigned short)num1;

						if( item->childCount() > 0 )
							subIndex = 0xFF;
						else
							subIndex = 0x00;
					}
				}
			}
		}
		populateAttributesForm( index, subIndex );
	}
}

void CMDIWindow::returnKeyPressed( QListViewItem* item )
{
	if( item )
	{
		// handle return key like mouse left-click
		mouseLeftPressed( item );

		// toggle visibility of the children, if any
		if( item->childCount() > 0 )
		{
			item->setOpen( !item->isOpen() );
		}
	}
}

void CMDIWindow::mouseRightClicked( QListViewItem* item, const QPoint& point )
{
	//@ToDo: handle mouse right-click
	//...
	if( item )
		if( point.isNull() )
			return;
}

void CMDIWindow::buttonSendClicked()
{
	// handle Send button
	if( (m_nodeId < 1) || (127 < m_nodeId) || !m_CANopen )
	{
		QMessageBox::warning( this, PROGRAM_STRING" - Send parameter",
								   "Data access is not possible in off-line mode!\n\n" \
								   "Close the document and connect to a device." );
		return;
	}
	if( (m_index == 0x0000) || (m_subIndex == 0xFF) )
	{
		QMessageBox::warning( this, PROGRAM_STRING" - Send parameter",
								   "No parameter is selected!\n\n" \
								   "Select one and try again." );
		return;
	}
	int o = m_dictionary.find( m_index );
	ASSERT( (0 <= o ) && (o < (int)m_dictionary.m_objectArray.count()) );

	CCANopenObject* ptrObject = m_dictionary.m_objectArray[o];
	ASSERT( ptrObject );

	int e = ptrObject->find( m_subIndex );
	ASSERT( (0 <= e ) && (e < (int)ptrObject->m_objectEntryArray.count()) );

	CCANopenObjectEntry* ptrEntry = ptrObject->m_objectEntryArray[e];
	ASSERT( ptrEntry );

	SDO_t_INTEGER8 sint8;
	SDO_t_INTEGER16 sint16;
	SDO_t_INTEGER32 sint32;
	SDO_t_UNSIGNED8 uint8;
	SDO_t_UNSIGNED16 uint16;
	SDO_t_UNSIGNED32 uint32;
	SDO_t_TIME_OF_DAY tod;
	SDO_t_DATE date;
	char string[256+1];
	//@ToDo:BYTE domain[1024];
	SHORT length;
	QString value = "";
	QString msg;
	int err = 0;

	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	switch( ptrEntry->getDataType() )
	{
	case CCANopenObjectEntry::typeInteger8:			//   Integer8
		if( CCANopenTypes::ascii2integer8( m_dataAccess.m_inputLineEdit->text().ascii(), &sint8) )
		{
			if( m_CANopen->writeInteger8( m_nodeId, m_index, m_subIndex, sint8 ) )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%+i", (CHAR)sint8 );
					break;
				case modeOct:
					sprintf( string, "%03o", (BYTE)sint8 );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%02X", (BYTE)sint8 );
					break;
				}
				value = string;
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
	case CCANopenObjectEntry::typeInteger16:		//   Integer16
		if( CCANopenTypes::ascii2integer16( m_dataAccess.m_inputLineEdit->text().ascii(), &sint16) )
		{
			if( m_CANopen->writeInteger16( m_nodeId, m_index, m_subIndex, sint16 ) )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%+i", (SHORT)sint16 );
					break;
				case modeOct:
					sprintf( string, "%06o", (WORD)sint16 );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%04X", (WORD)sint16 );
					break;
				}
				value = string;
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
	case CCANopenObjectEntry::typeInteger32:		//   Integer32
		if( CCANopenTypes::ascii2integer32( m_dataAccess.m_inputLineEdit->text().ascii(), &sint32) )
		{
			if( m_CANopen->writeInteger32( m_nodeId, m_index, m_subIndex, sint32 ) )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%+li", (LONG)sint32 );
					break;
				case modeOct:
					sprintf( string, "%011lo", (DWORD)sint32 );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%08lX", (DWORD)sint32 );
					break;
				}
				value = string;
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
	case CCANopenObjectEntry::typeUnsigned8:		//   Unsigned8
		if( CCANopenTypes::ascii2unsigned8( m_dataAccess.m_inputLineEdit->text().ascii(), &uint8) )
		{
			if( m_CANopen->writeUnsigned8( m_nodeId, m_index, m_subIndex, uint8 ) )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%u", uint8 );
					break;
				case modeOct:
					sprintf( string, "%03o", uint8 );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%02X", uint8 );
					break;
				}
				value = string;
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
	case CCANopenObjectEntry::typeUnsigned16:		//   Unsigned16
		if( CCANopenTypes::ascii2unsigned16( m_dataAccess.m_inputLineEdit->text().ascii(), &uint16) )
		{
			if( m_CANopen->writeUnsigned16( m_nodeId, m_index, m_subIndex, uint16 ) )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%u", uint16 );
					break;
				case modeOct:
					sprintf( string, "%06o", uint16 );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%04X", uint16 );
					break;
				}
				value = string;
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
	case CCANopenObjectEntry::typeUnsigned32:		//   Unsigned32
		if( CCANopenTypes::ascii2unsigned32( m_dataAccess.m_inputLineEdit->text().ascii(), &uint32) )
		{
			if( m_CANopen->writeUnsigned32( m_nodeId, m_index, m_subIndex, uint32 ) )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%lu", uint32 );
					break;
				case modeOct:
					sprintf( string, "%011lo", uint32 );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%08lX", uint32 );
					break;
				}
				value = string;
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
	case CCANopenObjectEntry::typeVisibleString:	//   Visible String
		if((length = m_dataAccess.m_inputLineEdit->text().length()) > 256)
			length = 256;
		for(short i = 0; i < length; i++)
			string[i] = m_dataAccess.m_inputLineEdit->text().at(i).latin1();
		if( m_CANopen->write( m_nodeId, m_index, m_subIndex, length, (BYTE*)&string[0] ) )
		{
			string[length] = 0;
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeTimeOfDay:
		if(CCANopenTypes::ascii2timeOfDay( m_dataAccess.m_inputLineEdit->text().ascii(), &tod))
		{
			if( m_CANopen->write( m_nodeId, m_index, m_subIndex, 6, (BYTE*)&tod.data[0] ) )
			{
				if( m_dataAccess.m_displayMode == modeDec )
				{
					CCANopenTypes::timeOfDay2ascii( tod, value );
				}
				else
				{
					for( int i = 0; i < 6; i++ )
					{
						switch( m_dataAccess.m_displayMode )
						{
						case modeDec:
							sprintf( string, "%u ", tod.data[i] );
							break;
						case modeOct:
							sprintf( string, "%03o ", tod.data[i] );
							break;
						case modeHex:
						default:
							sprintf( string, "0x%02X ", tod.data[i] );
							break;
						}
						value += string;
					}
				}
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
#ifndef _UNICODE_STRING
	case CCANopenObjectEntry::typeDate:
		if(CCANopenTypes::ascii2date( m_dataAccess.m_inputLineEdit->text().ascii(), &date))
		{
			if( m_CANopen->write( m_nodeId, m_index, m_subIndex, 7, (BYTE*)&date.data[0] ) )
			{
				if( m_dataAccess.m_displayMode == modeDec )
				{
					CCANopenTypes::date2ascii( date, value );
				}
				else
				{
					for( int i = 0; i < 7; i++ )
					{
						switch( m_dataAccess.m_displayMode )
						{
						case modeDec:
							sprintf( string, "%u ", date.data[i] );
							break;
						case modeOct:
							sprintf( string, "%03o ", date.data[i] );
							break;
						case modeHex:
						default:
							sprintf( string, "0x%02X ", date.data[i] );
							break;
						}
						value += string;
					}
				}
			}
			else
			{
				if( m_CANopen->isSDOAborted() )
					msg = QString( "Parameter could not be sent.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
				else
					msg = QString( "Parameter could not be sent.\n\nError code %1" ).arg( m_CANopen->getLastError() );
			}
		}
		else
		{
			msg = "Invalid data entered.\n\nPlease, try again!";
			err = TRUE;
		}
		break;
#endif
	case CCANopenObjectEntry::typeOctetString:
	default:
		msg = "Specified data type is not supported.\n\nSorry, not realized yet!";
	}
	QApplication::restoreOverrideCursor();

	if( m_CANopen->getLastError() || err )
	{
		QMessageBox::warning( this, PROGRAM_STRING" - Send parameter", msg );

		msg = QString( "Parameter '%1' could not be sent to node #%2" ).arg( ptrEntry->getName() ).arg( m_nodeId );
	}
	else
	{
		m_dataAccess.m_outputLineEdit->setText( value );

		msg = QString( "Parameter '%1' successfully sent to node #%2" ).arg( ptrEntry->getName() ).arg( m_nodeId );
	}
	emit( message( msg, 2000 ) );
}

void CMDIWindow::buttonReadClicked()
{
	// handle Read button
	if( (m_nodeId < 1) || (127 < m_nodeId) || !m_CANopen )
	{
		QMessageBox::warning( this, PROGRAM_STRING" - Read parameter",
								   "Data access is not possible in off-line mode!\n\n" \
								   "Close the document and connect to a device." );
		return;
	}
	if( (m_index == 0x0000) || (m_subIndex == 0xFF) )
	{
		QMessageBox::warning( this, PROGRAM_STRING" - Read parameter",
								   "No parameter is selected!\n\n" \
								   "Select one and try again." );
		return;
	}
	int o = m_dictionary.find( m_index );
	ASSERT( (0 <= o ) && (o < (int)m_dictionary.m_objectArray.count()) );

	CCANopenObject* ptrObject = m_dictionary.m_objectArray[o];
	ASSERT( ptrObject );

	int e = ptrObject->find( m_subIndex );
	ASSERT( (0 <= e ) && (e < (int)ptrObject->m_objectEntryArray.count()) );

	CCANopenObjectEntry* ptrEntry = ptrObject->m_objectEntryArray[e];
	ASSERT( ptrEntry );

	if( ptrEntry->getAccess() == CCANopenObjectEntry::accessWO )
	{
		m_dataAccess.m_outputLineEdit->setText( "" );
		return;
	}
	SDO_t_INTEGER8 sint8;
	SDO_t_INTEGER16 sint16;
	SDO_t_INTEGER32 sint32;
	SDO_t_UNSIGNED8 uint8;
	SDO_t_UNSIGNED16 uint16;
	SDO_t_UNSIGNED32 uint32;
	SDO_t_TIME_OF_DAY tod;
	SDO_t_DATE date;
	char string[256+1];
	BYTE domain[1024];
	SHORT length;
	QString value = "";
	QString msg;

	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	switch( ptrEntry->getDataType() )
	{
	case CCANopenObjectEntry::typeInteger8:			//   Integer8
		if( m_CANopen->readInteger8( m_nodeId, m_index, m_subIndex, &sint8 ) )
		{
			switch( m_dataAccess.m_displayMode )
			{
			case modeDec:
				sprintf( string, "%+i", (CHAR)sint8 );
				break;
			case modeOct:
				sprintf( string, "%03o", (BYTE)sint8 );
				break;
			case modeHex:
			default:
				sprintf( string, "0x%02X", (BYTE)sint8 );
				break;
			}
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeInteger16:		//   Integer16
		if( m_CANopen->readInteger16( m_nodeId, m_index, m_subIndex, &sint16 ) )
		{
			switch( m_dataAccess.m_displayMode )
			{
			case modeDec:
				sprintf( string, "%+i", (SHORT)sint16 );
				break;
			case modeOct:
				sprintf( string, "%06o", (WORD)sint16 );
				break;
			case modeHex:
			default:
				sprintf( string, "0x%04X", (WORD)sint16 );
				break;
			}
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeInteger32:		//   Integer32
		if( m_CANopen->readInteger32( m_nodeId, m_index, m_subIndex, &sint32 ) )
		{
			switch( m_dataAccess.m_displayMode )
			{
			case modeDec:
				sprintf( string, "%+li", (LONG)sint32 );
				break;
			case modeOct:
				sprintf( string, "%06lo", (DWORD)sint32 );
				break;
			case modeHex:
			default:
				sprintf( string, "0x%011lX", (DWORD)sint32 );
				break;
			}
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;

	case CCANopenObjectEntry::typeUnsigned8:		//   Unsigned8
		if( m_CANopen->readUnsigned8( m_nodeId, m_index, m_subIndex, &uint8 ) )
		{
			switch( m_dataAccess.m_displayMode )
			{
			case modeDec:
				sprintf( string, "%u", uint8 );
				break;
			case modeOct:
				sprintf( string, "%03o", uint8 );
				break;
			case modeHex:
			default:
				sprintf( string, "0x%02X", uint8 );
				break;
			}
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeUnsigned16:		//   Unsigned16
		if( m_CANopen->readUnsigned16( m_nodeId, m_index, m_subIndex, &uint16 ) )
		{
			switch( m_dataAccess.m_displayMode )
			{
			case modeDec:
				sprintf( string, "%u", uint16 );
				break;
			case modeOct:
				sprintf( string, "%06o", uint16 );
				break;
			case modeHex:
			default:
				sprintf( string, "0x%04X", uint16 );
				break;
			}
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeUnsigned32:		//   Unsigned32
		if( m_CANopen->readUnsigned32( m_nodeId, m_index, m_subIndex, &uint32 ) )
		{
			switch( m_dataAccess.m_displayMode )
			{
			case modeDec:
				sprintf( string, "%lu", uint32 );
				break;
			case modeOct:
				sprintf( string, "%011lo", uint32 );
				break;
			case modeHex:
			default:
				sprintf( string, "0x%08lX", uint32 );
				break;
			}
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeVisibleString:	//   Visible String
		if( m_CANopen->read( m_nodeId, m_index, m_subIndex, &length, (BYTE*)&string[0], 256 ) )
		{
			string[length] = 0;
			value = string;
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeTimeOfDay:		//   Time Of Day
		if( m_CANopen->read( m_nodeId, m_index, m_subIndex, &length, (BYTE*)&tod.data[0], 6 ) )
		{
			if( m_dataAccess.m_displayMode == modeDec )
			{
				CCANopenTypes::timeOfDay2ascii( tod, value );
			}
			else
			{
				for( int i = 0; i < length; i++ )
				{
					switch( m_dataAccess.m_displayMode )
					{
					case modeDec:
						sprintf( string, "%u ", tod.data[i] );
						break;
					case modeOct:
						sprintf( string, "%03o ", tod.data[i] );
						break;
					case modeHex:
					default:
						sprintf( string, "0x%02X ", tod.data[i] );
						break;
					}
					value += string;
				}
			}
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	case CCANopenObjectEntry::typeOctetString:		//   Octet String
		if( m_CANopen->read( m_nodeId, m_index, m_subIndex, &length, (BYTE*)&domain[0], 1024 ) )
		{
			for( int i = 0; i < length; i++ )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%u ", domain[i] );
					break;
				case modeOct:
					sprintf( string, "%03o ", domain[i] );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%02X ", domain[i] );
					break;
				}
				value += string;
			}
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
#ifndef _UNICODE_STRING								//   #DS-301, V4.0
	case CCANopenObjectEntry::typeDate:				//    Date
		if( m_CANopen->read( m_nodeId, m_index, m_subIndex, &length, (BYTE*)&date.data[0], 7 ) )
		{
			if( m_dataAccess.m_displayMode == modeDec )
			{
				CCANopenTypes::date2ascii( date, value );
			}
			else
			{
				for( int i = 0; i < length; i++ )
				{
					switch( m_dataAccess.m_displayMode )
					{
					case modeDec:
						sprintf( string, "%u ", date.data[i] );
						break;
					case modeOct:
						sprintf( string, "%03o ", date.data[i] );
						break;
					case modeHex:
					default:
						sprintf( string, "0x%02X ", date.data[i] );
						break;
					}
					value += string;
				}
			}
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
#endif												//   #for compatibility!
	case CCANopenObjectEntry::typeTimeDifference:	//   Time Difference
	case CCANopenObjectEntry::typeDomain:			//   Domain
	case CCANopenObjectEntry::typeInteger24:		//   Integer24
	case CCANopenObjectEntry::typeReal64:			//   Real64
	case CCANopenObjectEntry::typeInteger40:		//   Integer40
	case CCANopenObjectEntry::typeInteger48:		//   Integer48
	case CCANopenObjectEntry::typeInteger56:		//   Integer56
	case CCANopenObjectEntry::typeInteger64:		//   Integer64
	case CCANopenObjectEntry::typeUnsigned24:		//   Unsigned24
	case CCANopenObjectEntry::typeUnsigned40:		//   Unsigned40
	case CCANopenObjectEntry::typeUnsigned48:		//   Unsigned48
	case CCANopenObjectEntry::typeUnsigned56:		//   Unsigned56
	case CCANopenObjectEntry::typeUnsigned64:		//   Unsigned64
	case CCANopenObjectEntry::typeVoid:				//   'universal' data type
	case CCANopenObjectEntry::typeBoolean:			//   Boolean
	case CCANopenObjectEntry::typeReal32:			//   Real32
	default:
		if( m_CANopen->read( m_nodeId, m_index, m_subIndex, &length, (BYTE*)&domain[0], 1024 ) )
		{
			for( int i = 0; i < length; i++ )
			{
				switch( m_dataAccess.m_displayMode )
				{
				case modeDec:
					sprintf( string, "%u ", domain[i] );
					break;
				case modeOct:
					sprintf( string, "%03o ", domain[i] );
					break;
				case modeHex:
				default:
					sprintf( string, "0x%02X ", domain[i] );
					break;
				}
				value += string;
			}
		}
		else
		{
			if( m_CANopen->isSDOAborted() )
				msg = QString( "Parameter could not be read.\n\nSDO transfer aborted: 0x%1" ).arg( m_CANopen->getLastAbort(), 0, 16 );
			else
				msg = QString( "Parameter could not be read.\n\nError code %1" ).arg( m_CANopen->getLastError() );
		}
		break;
	}
	m_dataAccess.m_outputLineEdit->setText( value );
	QApplication::restoreOverrideCursor();

	if( m_CANopen->getLastError() )
	{
		QMessageBox::warning( this, PROGRAM_STRING" - Read parameter", msg );

		msg = QString( "Parameter '%1' could not be read from node #%2" ).arg( ptrEntry->getName() ).arg( m_nodeId );
	}
	else
	{
		msg = QString( "Parameter '%1' successfully read from node #%2" ).arg( ptrEntry->getName() ).arg( m_nodeId );
	}
	emit( message( msg, 2000 ) );
}

void CMDIWindow::buttonHexClicked()
{
	// set Hex mode for output
	m_dataAccess.m_displayMode = CMDIWindow::modeHex;

	// save settings
	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	settings.writeEntry( QString( "/%1/%2/%3" )
			.arg( APPLICATION_KEY )
			.arg( SETTINGS_KEY )
			.arg( DISPLAY_MODE_KEY ), (int)m_dataAccess.m_displayMode );

	// display selection
	m_dataAccess.m_hexRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeHex );
	m_dataAccess.m_decRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeDec );
	m_dataAccess.m_octRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeOct );

	// update output data, if online
	if( (1 <= m_nodeId) && (m_nodeId <= 127) )
	{
		if( (m_index != 0x0000) && (m_subIndex != 0xFF) )
		{
			buttonReadClicked();
		}
	}
}

void CMDIWindow::buttonDecClicked()
{
	// set Dec mode for output
	m_dataAccess.m_displayMode = CMDIWindow::modeDec;

	// save settings
	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	settings.writeEntry( QString( "/%1/%2/%3" )
			.arg( APPLICATION_KEY )
			.arg( SETTINGS_KEY )
			.arg( DISPLAY_MODE_KEY ), (int)m_dataAccess.m_displayMode );

	// display selection
	m_dataAccess.m_hexRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeHex );
	m_dataAccess.m_decRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeDec );
	m_dataAccess.m_octRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeOct );

	// update output data, if online
	if( (1 <= m_nodeId) && (m_nodeId <= 127) )
	{
		if( (m_index != 0x0000) && (m_subIndex != 0xFF) )
		{
			buttonReadClicked();
		}
	}
}

void CMDIWindow::buttonOctClicked()
{
	// set Oct mode for output
	m_dataAccess.m_displayMode = CMDIWindow::modeOct;

	// save settings
	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	settings.writeEntry( QString( "/%1/%2/%3" )
			.arg( APPLICATION_KEY )
			.arg( SETTINGS_KEY )
			.arg( DISPLAY_MODE_KEY ), (int)m_dataAccess.m_displayMode );

	// display selection
	m_dataAccess.m_hexRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeHex );
	m_dataAccess.m_decRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeDec );
	m_dataAccess.m_octRadioButton->setChecked( m_dataAccess.m_displayMode == CMDIWindow::modeOct );

	// update output data, if online
	if( (1 <= m_nodeId) && (m_nodeId <= 127) )
	{
		if( (m_index != 0x0000) && (m_subIndex != 0xFF) )
		{
			buttonReadClicked();
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CMDIWindow::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
