////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/MDIWindow.h $
//
//	project   :  COPbrowser - CANopen Object Browser
//	
//	purpose   :  MDIWindow - MDI (Multi Document Interface) Window
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CMDIWindow
//	
//	includes  :  dic/CANopen.h
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	MDIWindow - MDI (Multi Document Interface) Window

#ifndef _MDIWindow_h_
#define _MDIWindow_h_

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include <qmainwindow.h>
#include "dic/CANopen.h"

//////////////////////////////////////////////////////////////////////////////
//	forwarder

class QString;
class QListView;
class QListViewItem;
class QGroupBox;
class QLineEdit;
class QCheckBox;
class QPushButton;
class QRadioButton;
class CCANopenAccess;

//////////////////////////////////////////////////////////////////////////////
//	class CMDIWindow

class CMDIWindow : public QMainWindow
{
	Q_OBJECT
	
	//	enumerations:
	private:
		enum TDisplayMode {
			modeHex = 0,
			modeDec,
			modeOct
		};
	//	attributes:
	private:
		QString m_fileName;
		CCANopen m_dictionary;
		QListView* m_dictionaryListView;
		QGroupBox* m_objectGroupBox;
		QGroupBox* m_entryGroupBox;
		QGroupBox* m_accessGroupBox;
		struct {
			QLineEdit* m_nameLineEdit;
			QLineEdit* m_codeLineEdit;
			QLineEdit* m_typeLineEdit;
			QCheckBox* m_compoundCheckBox;
			QCheckBox* m_mandatoryCheckBox;
		} m_objectDescription;
		struct {
			QLineEdit* m_nameLineEdit;
			QLineEdit* m_typeLineEdit;
			QLineEdit* m_accessLineEdit;
			QLineEdit* m_rangeLineEdit;
			QLineEdit* m_defaultLineEdit;
			QCheckBox* m_mandatoryCheckBox;
			QCheckBox* m_mappingCheckBox;
		} m_entryDescription;
		struct {
			QLineEdit* m_inputLineEdit;
			QLineEdit* m_outputLineEdit;
			QPushButton* m_sendPushButton;
			QPushButton* m_readPushButton;
			QRadioButton* m_hexRadioButton;
			QRadioButton* m_decRadioButton;
			QRadioButton* m_octRadioButton;
			TDisplayMode m_displayMode;
		} m_dataAccess;
		CCANopenAccess* m_CANopen;
		unsigned char m_nodeId;
		unsigned short m_index;
		unsigned char m_subIndex;
	
	//	constructor/destructor:
	public:
		CMDIWindow( QWidget* parent = 0, const char* name = 0, WFlags wflags = WType_TopLevel );
		~CMDIWindow();
		
	//	member functions:
	public:
		bool openDocument( const QString& fileName, CCANopenAccess* CANopen = NULL );
		void showDeviceInfo();
	
	protected:
		void closeEvent( QCloseEvent* );
	
	private:
		void populateDictionaryList();
		//void clearDictionaryList();
		void populateAttributesForm( unsigned short index, unsigned char subIndex );
		void clearAttributesForm( bool entryOnly = false );
	
	private slots:
		void mouseLeftPressed( QListViewItem* );
		void returnKeyPressed( QListViewItem* );
		void mouseRightClicked( QListViewItem*, const QPoint& );
		void buttonSendClicked();
		void buttonReadClicked();
		void buttonHexClicked();
		void buttonDecClicked();
		void buttonOctClicked();
	
	signals:
		void message( const QString&, int );
	
	//	revision control:
	public:
		static const char* version();
};

#endif // _MDIWindow_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
