////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopen.cpp $
//
//	project   :  CAN - Controller Area Network
//
//	purpose   :  CANopen Object Dictionary
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopen
//
//	includes  :  CANopen.h (CANopenDeviceInfo.h, CANopenObject.h)
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Object Dictionary

static const char _id[] = "$Id: CANopen.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopen.h"

//////////////////////////////////////////////////////////////////////////////
//	class CCANopen

CCANopen::CCANopen()
{
	m_fileName = "";					// Name of the CANopen input file
	m_fileType = typeUnknown;			// Type of the CANopen input file
	m_fileState= CCANopen::statePublic;	// Dokumentstatus
	m_fileTitle = "CANopen Objektverzeichnis";
	m_fileVersion= CANopenXmlDtdVersion;

	m_info.m_author = "";				// Autor
	m_info.m_company = "";				// Firma
	m_info.m_revision = "1.0";			// Version
	m_info.m_date = "";					// Datum
	m_info.m_time = "";					// Uhrzeit
	m_info.m_URL = "";					// Internet URL
	m_info.m_mail = "";					// E-Mail Adresse

	m_deviceInfo = NULL;				// Devive info (empty)
}

CCANopen::~CCANopen()
{
	//@ToDo: tidy up!
}

int CCANopen::find(unsigned short index)
{
	// search object by index (result -1 = none)
	for( unsigned i = 0; i < m_objectArray.count(); i++ )
	{
		if( m_objectArray[i]->getIndex() == index )
			return (int)i;
	}
	return -1;
}

int CCANopen::find(const char* name)
{
	// search object by name (result -1 = none)
	for( unsigned i = 0; i < m_objectArray.count(); i++ )
	{
		if( m_objectArray[i]->getName().compare( name ) == 0 )
			return (int)i;
	}
	return -1;
}

QString CCANopen::getStateByIndex( TFileState index )
{
	QString strState;

	switch(index)						// State of an object:
	{
		case CCANopen::statePublic:	//   Public object
			strState = "Public";
			break;
		case CCANopen::stateProtected://   Protected object
			strState = "Protected";
			break;
		case CCANopen::statePrivate:	//   Private object
			strState = "Private";
			break;
		case CCANopen::stateInvalid:	//   Invalid object
			strState = "Invalid";
			break;
		default:							//   Sollte eigentlich nie vorkommen!
			strState = "Invalid";
			break;
	}
	return strState;
}

int CCANopen::getStateByName( const char* name )
{
	if(!strcasecmp(name, "Public"))		//   Public object
		return (int)CCANopen::statePublic;
	else if(!strcasecmp(name, "Protected"))// Protected object
		return (int)CCANopen::stateProtected;
	else if(!strcasecmp(name, "Private"))//   Private object
		return (int)CCANopen::statePrivate;
	else								//   Invalid object
		return (int)CCANopen::stateInvalid;
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CCANopen::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
