////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopen.h $
//
//	project   :  CAN - Controller Area Network
//	
//	purpose   :  CANopen Object Dictionary
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopen
//	
//	includes  :  CANopenDeviceInfo.h, CANopenObject.h
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Object Dictionary

#ifndef _CANopen_h_
#define _CANopen_h_

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenDeviceInfo.h"
#include "CANopenObject.h"
#include <qstring.h>
#include <qptrvector.h>

//////////////////////////////////////////////////////////////////////////////
//	defines

#define CANopenXmlDtdVersion  "1.0"		// Document Type Definition (revision)

//////////////////////////////////////////////////////////////////////////////
//	class CCANopen

class CCANopen
{
	friend class CCANopenReader;		// access by class CCANopenReader allowed
	
	//	enumerations:
	public:
		enum TFileType {				// Supported file types:
			typeEDS = 0,				//   EDS (CiA - Electronic Data Sheet)
			typeDCF,					//   DCF (CiA - Device Configuration File)
			typeXML,					//   XML (UVS - eXtensible Markup Language)
			typeCOP,					//   COP (UVS - CANopen Objektedatei, ASCII)
			typeUnknown = -1			//   *.* (unknown file type)
		};
		enum TFileState {				// Dokumentstatus:
			statePublic = 0,			//   Öffentliche Version
			stateProtected,				//   Firmen-interne Version
			statePrivate,				//   Entwicklerversion
			stateInvalid = -1			//   Ungültige Version
		};
	//	attributes:
	private:
		QString		m_fileName;			// Name of the CANopen input file
		TFileType	m_fileType;			// Type of the CANopen input file
		TFileState	m_fileState;		// Dokumentstatus
		QString     m_fileTitle;		// Dokumenttitel
		QString     m_fileVersion;		// Dokumentversion
		struct tInfo {					// Dokumentinformationen:
			QString m_author;			//   Autor
			QString m_company;			//   Firma
			QString m_revision;			//   Version
			QString m_date;				//   Datum
			QString m_time;				//   Uhrzeit
			QString m_URL;				//   Internet URL
			QString m_mail;				//   E-Mail Adresse
			QString m_text;				//   Beschreibung
		}	m_info;
	
	public:
		CCANopenDeviceInfo* m_deviceInfo;			// Device info
		QPtrVector<CCANopenObject> m_objectArray;	// Object dictionary
		
	//	constructor/destructor:
	public:
		CCANopen();
		~CCANopen();
		
	//	member functions:
	public:
		QString getFileName()			{ return m_fileName; }
		QString getAuthor()				{ return m_info.m_author; };
		QString getCompany()			{ return m_info.m_company; };
		QString getRevision()			{ return m_info.m_revision; };
		QString getDate()				{ return m_info.m_date; };
		QString getTime()				{ return m_info.m_time; };
		QString getURL()				{ return m_info.m_URL; };
		QString getMail()				{ return m_info.m_mail; };
		TFileState getState()			{ return m_fileState; };
	
		int find( unsigned short index );	// search object by index (result -1 = none)
		int find( const char* name );		// search object by name (result -1 = none)
	
	//	static member
	public:
		static QString getStateByIndex( TFileState index );
		static int getStateByName( const char * name );

		static const char* getCopyright();		// Copyright-Vermerk des Modules!
	
	//	revision control:
	public:
		static const char* version();
};

#endif // _CANopen_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
