////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopenDeviceInfo.cpp $
//
//	project   :  CAN - Controller Area Network
//
//	purpose   :  CANopen Device Info
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenDeviceInfo
//
//	includes  :  CANopenDeviceInfo.h
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Device Info

static const char _id[] = "$Id: CANopenDeviceInfo.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenDeviceInfo.h"

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenDeviceInfo

CCANopenDeviceInfo::CCANopenDeviceInfo()
{
	m_vendorName = "";					// Name of the vendor
	m_productName = "";					// Name of the product
	m_vendorNumber = 0UL;				// LSS vendor-id.
	m_productNumber = 0UL;				// LSS product code
	m_revisionNumber = 0UL;				// LSS product revision
	m_orderCode = "";					// Order code for the product
	m_baudrate[baudrate1000] = false;	// Baudrate 1000 kBit/s
	m_baudrate[baudrate800] = false;	// Baudrate 800 kBit/s
	m_baudrate[baudrate500] = false;	// Baudrate 500 kBit/s
	m_baudrate[baudrate250] = false;	// Baudrate 250 kBit/s
	m_baudrate[baudrate125] = false;	// Baudrate 125 kBit/s
	m_baudrate[baudrate100] = false;	// Baudrate 100 kBit/s
	m_baudrate[baudrate50] = false;		// Baudrate 50 kBit/s
	m_baudrate[baudrate20] = false;		// Baudrate 20 kBit/s
	m_baudrate[baudrate10] = false;		// Baudrate 10 kBit/s
	m_simpleBootUp.m_master = false;	// Simple boot-up master
	m_simpleBootUp.m_slave = false;	// Simple boot-up slave
	m_PDO.m_rxPDO = 0;					// Number of receive PDOs
	m_PDO.m_txPDO = 0;					// Number of transmit PDOs
	m_PDO.m_granularity = 0;			// Granularity allowed for mapping
	m_dynamicChannels = false;			// Facility of dynamic variable generation
	m_groupMessaging = false;			// Facility of multiplexed PDOs
	m_layerSettings = false;			// Layer Settings Specification
	m_lmtManufacturerName = "";			// LMT manufacturer name (DS-301, V3.0)
	m_lmtProductName = "";				// LMT product name (DS-301, V3.0)
	m_EDSVersion = "4.0";				// Version of EDS-file specification ("4.0")
}

CCANopenDeviceInfo::~CCANopenDeviceInfo()
{
}

QString CCANopenDeviceInfo::getBaudrateByIndex( TBaudrateIndex index )
{
	QString strBaudrate;

	switch( index )						// Indexes for the baudrate-table:
	{
		case CCANopenDeviceInfo::baudrate1000:	// Baudrate 1000 kBit/s
			strBaudrate = "1000";
			break;
		case CCANopenDeviceInfo::baudrate800:	// Baudrate 800 kBit/s
			strBaudrate = "800";
			break;
		case CCANopenDeviceInfo::baudrate500:	// Baudrate 500 kBit/s
			strBaudrate = "500";
			break;
		case CCANopenDeviceInfo::baudrate250:	// Baudrate 250 kBit/s
			strBaudrate = "250";
			break;
		case CCANopenDeviceInfo::baudrate125:	// Baudrate 125 kBit/s
			strBaudrate = "125";
			break;
		case CCANopenDeviceInfo::baudrate100:	// Baudrate 100 kBit/s
			strBaudrate = "100";
			break;
		case CCANopenDeviceInfo::baudrate50:	// Baudrate 50 kBit/s
			strBaudrate = "50";
			break;
		case CCANopenDeviceInfo::baudrate20:	// Baudrate 20 kBit/s
			strBaudrate = "20";
			break;
		case CCANopenDeviceInfo::baudrate10:	// Baudrate 10 kBit/s
			strBaudrate = "10";
			break;
		default:							//   Sollte eigentlich nie vorkommen!
			strBaudrate = "";
			break;
	}
	return strBaudrate;
}

QString CCANopenDeviceInfo::getBooleanByValue(bool boolean)
{
	QString strBoolean;

	if( boolean )						// True or
		strBoolean = "Yes";
	else								// False
		strBoolean = "No";

	return strBoolean;
}

QString CCANopenDeviceInfo::getStateByIndex( TDeviceInfoState index )
{
	QString strDeviceInfoState;

	switch( index )						// State of the device-info:
	{
		case CCANopenDeviceInfo::statePublic:	//   Public device-info
			strDeviceInfoState = "Public";
			break;
		case CCANopenDeviceInfo::stateProtected://   Protected device-info
			strDeviceInfoState = "Protected";
			break;
		case CCANopenDeviceInfo::statePrivate:	//   Private device-info
			strDeviceInfoState = "Private";
			break;
		case CCANopenDeviceInfo::stateInvalid:	//   Invalid device-info
			strDeviceInfoState = "Invalid";
			break;
		default:							//   Sollte eigentlich nie vorkommen!
			strDeviceInfoState = "Invalid";
			break;
	}
	return strDeviceInfoState;
}

int CCANopenDeviceInfo::getBaudrateByName( const char* name )
{
	if(!strcmp(name, "1000"))		//   Baudrate 1000 kBit/s
		return (int)CCANopenDeviceInfo::baudrate1000;
	else if(!strcmp(name, "800"))	//   Baudrate 800 kBit/s
		return (int)CCANopenDeviceInfo::baudrate800;
	else if(!strcmp(name, "500"))	//   Baudrate 500 kBit/s
		return (int)CCANopenDeviceInfo::baudrate500;
	else if(!strcmp(name, "250"))	//   Baudrate 250 kBit/s
		return (int)CCANopenDeviceInfo::baudrate250;
	else if(!strcmp(name, "125"))	//   Baudrate 125 kBit/s
		return (int)CCANopenDeviceInfo::baudrate125;
	else if(!strcmp(name, "100"))	//   Baudrate 100 kBit/s
		return (int)CCANopenDeviceInfo::baudrate100;
	else if(!strcmp(name, "50"))		//   Baudrate 50 kBit/s
		return (int)CCANopenDeviceInfo::baudrate50;
	else if(!strcmp(name, "20"))		//   Baudrate 20 kBit/s
		return (int)CCANopenDeviceInfo::baudrate20;
	else if(!strcmp(name, "10"))		//   Baudrate 10 kBit/s
		return (int)CCANopenDeviceInfo::baudrate10;
	else
		return (int)-1;
}

bool CCANopenDeviceInfo::getBooleanByName( const char* name )
{
	if(!strcmp(name, "True"))
		return true;
	else if(!strcmp(name, "Yes"))
		return true;
	else if(!strcmp(name, "1"))
		return true;
	else
		return false;
}

int CCANopenDeviceInfo::getStateByName( const char* name )
{
	if(!strcmp(name, "Public"))		//   Public device-info
		return (int)CCANopenDeviceInfo::statePublic;
	else if(!strcmp(name, "Protected"))// Protected device-info
		return (int)CCANopenDeviceInfo::stateProtected;
	else if(!strcmp(name, "Private"))//   Private device-info
		return (int)CCANopenDeviceInfo::statePrivate;
	else								//   Invalid device-info
		return (int)CCANopenDeviceInfo::stateInvalid;
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CCANopenDeviceInfo::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
