////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopenObject.cpp $
//
//	project   :  CAN - Controller Area Network
//
//	purpose   :  CANopen Object
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenObject
//
//	includes  :  CANopenObject.h (CANopenObjectEntry.h)
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	<description>

static const char _id[] = "$Id: CANopenObject.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenObject.h"

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenObject

CCANopenObject::CCANopenObject( unsigned short index )
{
	m_index = index;					// Profile index number
	m_name = "";						// Name of the object
	m_class = classNULL;				// Variable classification
	m_dataType = typeVoid;				// Data type classification
	m_category = categoryOptional;		// 'Optional' or 'Mandatory' object
}

CCANopenObject::CCANopenObject()
{
	m_index = 0x0000;					// Profile index number
	m_name = "";						// Name of the object
	m_class = classNULL;				// Variable classification
	m_dataType = typeVoid;				// Data type classification
	m_category = categoryOptional;		// 'Optional' or 'Mandatory' object
}

CCANopenObject::~CCANopenObject()
{
}

QString CCANopenObject::getProfile( bool link )
{
	QString profile;

	if(!this->m_profile.m_number.isEmpty())
	{
		profile = this->m_profile.m_number;

		if(!this->m_profile.m_version.isEmpty())
		{
			profile += ", ";
			profile += this->m_profile.m_version;
		}
		if(!this->m_profile.m_page.isEmpty())
		{
			profile += ", Page ";
			profile += this->m_profile.m_page;
		}
		if(!this->m_profile.m_chapter.isEmpty())
		{
			profile += ", ";
			profile += this->m_profile.m_chapter;
		}
		if(!this->m_profile.m_link.isEmpty() && link)
		{
			profile += ", <a href=\"";
			profile += this->m_profile.m_link;
			profile += "\">";
			profile += this->m_profile.m_link;
			profile += "</a>";
		}
	}
	else
	{
		profile = "";
	}
	return profile;
}

bool CCANopenObject::isCommunicationObject()
{
	// Objekt ist aus dem Kommunikationsprofil?
	return (bool)((0x1000 <= this->m_index) && (this->m_index <= 0x1FFF));
}

bool CCANopenObject::isDeviceProfileObject()
{
	// Objekt ist aus einem Geräteprofil?
	return (bool)((0x6000 <= this->m_index) && (this->m_index <= 0x9FFF));
}

bool CCANopenObject::isManufacturerSpecificObject()
{
	// Objekt ist Hersteller-spezifisch?
	return (bool)((0x2000 <= this->m_index) && (this->m_index <= 0x5FFF));
}

bool CCANopenObject::isSingleObject()
{
	// Objekt ist eine VARiable?
	return (bool)(this->m_class == CCANopenObject::classVAR);
}

bool CCANopenObject::isMultipleObject()
{
	// Objekt ist ein ARRAY oder ein RECORD?
	return (bool)(this->m_class == CCANopenObject::classARRAY)
		|| (bool)(this->m_class == CCANopenObject::classRECORD);
}

bool CCANopenObject::isStandardDataType()
{
	// Objekt hat einen Basis-Datentyp?
	return (bool)((0x001 <= this->m_dataType) && (this->m_dataType <= 0x00D))
		|| (bool)((0x00F <= this->m_dataType) && (this->m_dataType <= 0x016))
		|| (bool)((0x018 <= this->m_dataType) && (this->m_dataType <= 0x01B))
		|| (bool)((0x060 <= this->m_dataType) && (this->m_dataType <= 0x7FF))
		|| (bool)((0x0A0 <= this->m_dataType) && (this->m_dataType <= 0x0BF))
		|| (bool)((0x0E0 <= this->m_dataType) && (this->m_dataType <= 0x0FF))
		|| (bool)((0x120 <= this->m_dataType) && (this->m_dataType <= 0x13F))
		|| (bool)((0x160 <= this->m_dataType) && (this->m_dataType <= 0x17F))
		|| (bool)((0x1A0 <= this->m_dataType) && (this->m_dataType <= 0x1BF))
		|| (bool)((0x1E0 <= this->m_dataType) && (this->m_dataType <= 0x1FF))
		|| (bool)((0x220 <= this->m_dataType) && (this->m_dataType <= 0x23F));
}

bool CCANopenObject::isComplexDataType()
{
	// Objekt hat einen strukturierten Datentyp?
	return (bool)((0x020 <= this->m_dataType) && (this->m_dataType <= 0x023))
		|| (bool)((0x040 <= this->m_dataType) && (this->m_dataType <= 0x05F))
		|| (bool)((0x080 <= this->m_dataType) && (this->m_dataType <= 0x09F))
		|| (bool)((0x0C0 <= this->m_dataType) && (this->m_dataType <= 0x0DF))
		|| (bool)((0x100 <= this->m_dataType) && (this->m_dataType <= 0x11F))
		|| (bool)((0x140 <= this->m_dataType) && (this->m_dataType <= 0x15F))
		|| (bool)((0x180 <= this->m_dataType) && (this->m_dataType <= 0x19F))
		|| (bool)((0x1C0 <= this->m_dataType) && (this->m_dataType <= 0x1DF))
		|| (bool)((0x200 <= this->m_dataType) && (this->m_dataType <= 0x21F))
		|| (bool)((0x240 <= this->m_dataType) && (this->m_dataType <= 0x25F));
}

int CCANopenObject::find( unsigned char subIndex )
{
	// search object entry by sub-index (result -1 = none)
	for( unsigned i = 0; i < m_objectEntryArray.count(); i++ )
	{
		if( m_objectEntryArray[i]->getSubindex() == subIndex )
			return (int)i;
	}
	return -1;
}

int CCANopenObject::find( const char* name )
{
	// search object entry by name (result -1 = none)
	for( unsigned i = 0; i < m_objectEntryArray.count(); i++ )
	{
		if( m_objectEntryArray[i]->getName().compare( name ) == 0 )
			return (int)i;
	}
	return -1;
}

QString CCANopenObject::getStateByIndex( TObjectState index )
{
	QString strObjectState;

	switch(index)						// State of an object:
	{
		case CCANopenObject::statePublic:	//   Public object
			strObjectState = "Public";
			break;
		case CCANopenObject::stateProtected://   Protected object
			strObjectState = "Protected";
			break;
		case CCANopenObject::statePrivate:	//   Private object
			strObjectState = "Private";
			break;
		case CCANopenObject::stateInvalid:	//   Invalid object
			strObjectState = "Invalid";
			break;
		default:							//   Sollte eigentlich nie vorkommen!
			strObjectState = "Invalid";
			break;
	}
	return strObjectState;
}

QString CCANopenObject::getClassByIndex( TObjectClass index )
{
	QString strObjectClass;

	switch(index)						// Object class of an object:
	{
		case CCANopenObject::classNULL:		//   NULL       = An entry with no data fields
			strObjectClass = "NULL";
			break;
		case CCANopenObject::classDOMAIN:	//   DOMAIN     = Large variable amount of data
			strObjectClass = "DOMAIN";
			break;
		case CCANopenObject::classDEFTYPE:	//   DEFTYPE    = Denotes a type definition
			strObjectClass = "DEFTYPE";
			break;
		case CCANopenObject::classDEFSTRUCT://   DEFSTRUCT  = Defines a new record type
			strObjectClass = "DEFSTRUCT";
			break;
		case CCANopenObject::classVAR:		//   VAR        = A single value
			strObjectClass = "VAR";
			break;
		case CCANopenObject::classARRAY:	//   ARRAY      = A multiple data field object of the same type
			strObjectClass = "ARRAY";
			break;
		case CCANopenObject::classRECORD:	//   RECORD     = A multiple data fiedd object of different types
			strObjectClass = "RECORD";
			break;
		default:							//   Sollte eigentlich nie vorkommen!
			strObjectClass = "NULL";
			break;
	}
	return strObjectClass;
}

QString CCANopenObject::getDataTypeByIndex( TObjectType index )
{
	QString strObjectType;

	switch(index)						// Type attribute of an object:
	{
		case CCANopenObject::typeBoolean:	//   Boolean
			strObjectType = "Boolean";
			break;
		case CCANopenObject::typeInteger8:	//   Integer8
			strObjectType = "Integer8";
			break;
		case CCANopenObject::typeInteger16:	//   Integer16
			strObjectType = "Integer16";
			break;
		case CCANopenObject::typeInteger32:	//   Integer32
			strObjectType = "Integer32";
			break;
		case CCANopenObject::typeUnsigned8:	//   Unsigned8
			strObjectType = "Unsigned8";
			break;
		case CCANopenObject::typeUnsigned16://   Unsigned16
			strObjectType = "Unsigned16";
			break;
		case CCANopenObject::typeUnsigned32://   Unsigned32
			strObjectType = "Unsigned32";
			break;
		case CCANopenObject::typeReal32:	//   Real32
			strObjectType = "Real32";
			break;
		case CCANopenObject::typeVisibleString://Visible String
			strObjectType = "Visible String";
			break;
		case CCANopenObject::typeOctetString://  Octet String
			strObjectType = "Octet String";
			break;
#ifdef _UNICODE_STRING
		case CCANopenObject::typeUnicodeString://Unicode String
			strObjectType = "Unicode String";
			break;
#else
		case CCANopenObject::typeDate:		// Date
			strObjectType = "Date";
			break;
#endif
		case CCANopenObject::typeTimeOfDay:	//   Time Of Day
			strObjectType = "Time Of Day";
			break;
		case CCANopenObject::typeTimeDifference://Time Difference
			strObjectType = "Time Difference";
			break;
		case CCANopenObject::typeDomain:	//   Domain
			strObjectType = "Domain";
			break;
		case CCANopenObject::typeInteger24:	//   Integer24
			strObjectType = "Integer24";
			break;
		case CCANopenObject::typeReal64:	//   Real64
			strObjectType = "Real64";
			break;
		case CCANopenObject::typeInteger40:	//   Integer40
			strObjectType = "Integer40";
			break;
		case CCANopenObject::typeInteger48:	//   Integer48
			strObjectType = "Integer48";
			break;
		case CCANopenObject::typeInteger56:	//   Integer56
			strObjectType = "Integer56";
			break;
		case CCANopenObject::typeInteger64:	//   Integer64
			strObjectType = "Integer64";
			break;
		case CCANopenObject::typeUnsigned24://   Unsigned24
			strObjectType = "Unsigned24";
			break;
		case CCANopenObject::typeUnsigned40://   Unsigned40
			strObjectType = "Unsigned40";
			break;
		case CCANopenObject::typeUnsigned48://   Unsigned48
			strObjectType = "Unsigned48";
			break;
		case CCANopenObject::typeUnsigned56://   Unsigned56
			strObjectType = "Unsigned56";
			break;
		case CCANopenObject::typeUnsigned64://   Unsigned64
			strObjectType = "Unsigned64";
			break;
		case CCANopenObject::typePDOParameter:// PDO CommPar
			strObjectType = "PDO CommPar";
			break;
		case CCANopenObject::typePDOMapping://   PDO Mapping
			strObjectType = "PDO Mapping";
			break;
		case CCANopenObject::typeSDOParameter:// SDO Parameter
			strObjectType = "SDO Parameter";
			break;
		case CCANopenObject::typeIdentity:	//   Identity
			strObjectType = "Identity";
			break;
		default:							//   Benutzer-definierter Datentyp
			strObjectType = "User-defined Type";
			break;
	}
	return strObjectType;
}

QString CCANopenObject::getCategoryByIndex( TObjectCategory index )
{
	QString strObjectCategory;

	switch(index)						// Category of an object:
	{
		case CCANopenObject::categoryOptional:		// Optional object
			strObjectCategory = "Optional";
			break;
		case CCANopenObject::categoryMandatory:		// Mandatory object
			strObjectCategory = "Mandatory";
			break;
		case CCANopenObject::categoryConditional:	// Conditional object
			strObjectCategory = "Conditional";
			break;
		default:							//   Sollte eigentlich nie vorkommen!
			strObjectCategory = "Optional";
			break;
	}
	return strObjectCategory;
}

int CCANopenObject::getStateByName( const char* name )
{
	if(!strcasecmp(name, "Public"))		//   Public object
		return (int)CCANopenObject::statePublic;
	else if(!strcasecmp(name, "Protected"))// Protected object
		return (int)CCANopenObject::stateProtected;
	else if(!strcasecmp(name, "Private"))//   Private object
		return (int)CCANopenObject::statePrivate;
	else								//   Invalid object
		return (int)CCANopenObject::stateInvalid;
}

int CCANopenObject::getClassByName( const char* name )
{
	if(!strcasecmp(name, "NULL"))     	//   NULL       = An entry with no data fields
		return (int)CCANopenObject::classNULL;
	else if(!strcasecmp(name, "DOMAIN")) //   DOMAIN     = Large variable amount of data
		return (int)CCANopenObject::classDOMAIN;
	else if(!strcasecmp(name, "DEFTYPE"))//   DEFTYPE    = Denotes a type definition
		return (int)CCANopenObject::classDEFTYPE;
	else if(!strcasecmp(name, "DEFSTRUCT"))// DEFSTRUCT  = Defines a new record type
		return (int)CCANopenObject::classDEFSTRUCT;
	else if(!strcasecmp(name, "VAR"))    //   VAR        = A single value
		return (int)CCANopenObject::classVAR;
	else if(!strcasecmp(name, "ARRAY"))  //   ARRAY      = A multiple data field object of the same type
		return (int)CCANopenObject::classARRAY;
	else if(!strcasecmp(name, "RECORD")) //   RECORD     = A multiple data fiedd object of different types
		return (int)CCANopenObject::classRECORD;
	else								//   Sollte eigentlich nie vorkommen!
		return (int)CCANopenObject::classNULL;
}

int CCANopenObject::getDataTypeByName( const char* name )
{
	if(!strcasecmp(name, "void"))
		return (int)CCANopenObject::typeVoid;
	else if(!strcasecmp(name, "Boolean"))
		return (int)CCANopenObject::typeBoolean;
	else if(!strcasecmp(name, "Integer8"))
		return (int)CCANopenObject::typeInteger8;
	else if(!strcasecmp(name, "Integer16"))
		return (int)CCANopenObject::typeInteger16;
	else if(!strcasecmp(name, "Integer32"))
		return (int)CCANopenObject::typeInteger32;
	else if(!strcasecmp(name, "Unsigned8"))
		return (int)CCANopenObject::typeUnsigned8;
	else if(!strcasecmp(name, "Unsigned16"))
		return (int)CCANopenObject::typeUnsigned16;
	else if(!strcasecmp(name, "Unsigned32"))
		return (int)CCANopenObject::typeUnsigned32;
	else if(!strcasecmp(name, "Real32"))
		return (int)CCANopenObject::typeReal32;
	else if(!strcasecmp(name, "Visible String"))
		return (int)CCANopenObject::typeVisibleString;
	else if(!strcasecmp(name, "Octet String"))
		return (int)CCANopenObject::typeOctetString;
#ifdef _UNICODE_STRING
	else if(!strcasecmp(name, "Unicode String"))
		return (int)CCANopenObject::typeUnicodeString;
#else
	else if(!strcasecmp(name, "Date"))
		return (int)CCANopenObject::typeDate;
#endif
	else if(!strcasecmp(name, "Time Of Day"))
		return (int)CCANopenObject::typeTimeOfDay;
	else if(!strcasecmp(name, "Time Difference"))
		return (int)CCANopenObject::typeTimeDifference;
	else if(!strcasecmp(name, "Domain"))
		return (int)CCANopenObject::typeDomain;
	else if(!strcasecmp(name, "Integer24"))
		return (int)CCANopenObject::typeInteger24;
	else if(!strcasecmp(name, "Real64"))
		return (int)CCANopenObject::typeReal64;
	else if(!strcasecmp(name, "Integer40"))
		return (int)CCANopenObject::typeInteger40;
	else if(!strcasecmp(name, "Integer48"))
		return (int)CCANopenObject::typeInteger48;
	else if(!strcasecmp(name, "Integer56"))
		return (int)CCANopenObject::typeInteger56;
	else if(!strcasecmp(name, "Integer64"))
		return (int)CCANopenObject::typeInteger64;
	else if(!strcasecmp(name, "Unsigned24"))
		return (int)CCANopenObject::typeUnsigned24;
	else if(!strcasecmp(name, "Unsigned40"))
		return (int)CCANopenObject::typeUnsigned40;
	else if(!strcasecmp(name, "Unsigned48"))
		return (int)CCANopenObject::typeUnsigned48;
	else if(!strcasecmp(name, "Unsigned56"))
		return (int)CCANopenObject::typeUnsigned56;
	else if(!strcasecmp(name, "Unsigned64"))
		return (int)CCANopenObject::typeUnsigned64;
	else if(!strcasecmp(name, "PDO CommPar"))
		return (int)CCANopenObject::typePDOParameter;
	else if(!strcasecmp(name, "PDO Mapping"))
		return (int)CCANopenObject::typePDOMapping;
	else if(!strcasecmp(name, "SDO Parameter"))
		return (int)CCANopenObject::typeSDOParameter;
	else if(!strcasecmp(name, "Identity"))
		return (int)CCANopenObject::typeIdentity;
	else								//   Benutzer-definierter Datentyp
		//return (int)typeUserDefined;
		return (int)typeVoid;
}

int CCANopenObject::getCategoryByName( const char* name )
{
	if(!strncmp(name, "Optional", strlen("Optional")))
		return (int)CCANopenObject::categoryOptional;
	else if(!strncmp(name, "Mandatory", strlen("Mandatory")))
		return (int)CCANopenObject::categoryMandatory;
	else if(!strncmp(name, "Conditional", strlen("Conditional")))
		return (int)CCANopenObject::categoryConditional;
	else // Should not happen!
		return (int)CCANopenObject::categoryOptional;
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CCANopenObject::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
