////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopenObject.h $
//
//	project   :  CAN - Controller Area Network
//	
//	purpose   :  CANopen Object
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenObject
//	
//	includes  :  CANopenObjectEntry.h
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Object

#ifndef _CANopenObject_h_
#define _CANopenObject_h_

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenObjectEntry.h"
#include <qstring.h>
#include <qptrvector.h>

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenObject

class CCANopenObject
{
	//	enumerations:
	public:
		enum TObjectClass {				// Object class of an object:
			classNULL = 0,				//   NULL       = An entry with no data fields
			classDOMAIN = 2,			//   DOMAIN     = Large variable amount of data
			classDEFTYPE = 5,			//   DEFTYPE    = Denotes a type definition
			classDEFSTRUCT = 6,			//   DEFSTRUCT  = Defines a new record type
			classVAR = 7,				//   VAR        = A single value
			classARRAY = 8,				//   ARRAY      = A multiple data field object of the same type
			classRECORD = 9,			//   RECORD     = A multiple data fiedd object of different types
		};
		enum TObjectType {				// Type attribute of an object:
			typeBoolean = 0x1,			//   Boolean
			typeInteger8 = 0x2,			//   Integer8
			typeInteger16 = 0x3,		//   Integer16
			typeInteger32 = 0x4,		//   Integer32
			typeUnsigned8 = 0x5,		//   Unsigned8
			typeUnsigned16 = 0x6,		//   Unsigned16
			typeUnsigned32 = 0x7,		//   Unsigned32
			typeReal32 = 0x8,			//   Real32
			typeVisibleString = 0x9,	//   Visible String
			typeOctetString = 0xA,		//   Octet String
#ifdef _UNICODE_STRING					//   #DS-301, V4.0
			typeUnicodeString = 0xB,	//    Unicode String
#else									//   #DS-301, V3.0
			typeDate = 0xB,				//    Date
#endif									//   #for compatibility!
			typeTimeOfDay = 0xC,		//   Time Of Day
			typeTimeDifference = 0xD,	//   Time Difference
			typeDomain = 0xF,			//   Domain
			typeInteger24 = 0x10,		//   Integer24
			typeReal64 = 0x11,			//   Real64
			typeInteger40 = 0x12,		//   Integer40
			typeInteger48 = 0x13,		//   Integer48
			typeInteger56 = 0x14,		//   Integer56
			typeInteger64 = 0x15,		//   Integer64
			typeUnsigned24 = 0x16,		//   Unsigned24
			typeUnsigned40 = 0x18,		//   Unsigned40
			typeUnsigned48 = 0x19,		//   Unsigned48
			typeUnsigned56 = 0x1A,		//   Unsigned56
			typeUnsigned64 = 0x1B,		//   Unsigned64
			typePDOParameter = 0x20,	//   PDO CommPar
			typePDOMapping = 0x21,		//   PDO Mapping
			typeSDOParameter = 0x22,	//   SDO Parameter
			typeIdentity = 0x23,		//   Identity
			typeVoid = 0,				//   'universal' data type
			//typeUserDefined = -1,		//   User-defined data type
		};
		enum TObjectCategory {			// Category of an object:
			categoryOptional = 0,		//   Optional object
			categoryMandatory = 1,		//   Mandatory object
			categoryConditional,		//   Conditional object
		};
		enum TObjectState {				// State of an object:
			statePublic = 0,			//   Public  object
			stateProtected,				//   Protected object
			statePrivate,				//   Private object
			stateInvalid = -1,			//   Invalid object
		};
	//	attributes:
	private:
		unsigned short	m_index;		// Profile index number
		QString			m_name;			// Name of the object
		TObjectClass	m_class;		// Variable classification
		TObjectType		m_dataType;		// Data type classification
		TObjectCategory m_category;		// 'Optional' or 'Mandatory' object
	
	struct tProfile {					// Document of standardization (uv-extension):
		QString	m_number;				//   Designation of the document
		QString	m_version;				//   Revision number of the document
		QString	m_chapter;				//   Chapter within the designated document
		QString	m_page;					//   Page number within the designated document
		QString m_link;					//   A hyper-link to the designated document
	} m_profile;
	
	public:
		QPtrVector<CCANopenObjectEntry> m_objectEntryArray;// Object entries (= parameters)
	
	//	constructor/destructor:
	public:
		CCANopenObject( unsigned short index );
		~CCANopenObject();
	protected:
		CCANopenObject();
	
	//	member functions:
	public:
		unsigned short	getIndex()		{ return m_index; }
		QString			getName()		{ return m_name; }
		TObjectClass	getClass()		{ return m_class; };
		TObjectType		getDataType()	{ return m_dataType; };
		TObjectCategory getCategory()	{ return m_category; };
		QString	getProfileNumber()		{ return m_profile.m_number; };
		QString	getProfileVersion()		{ return m_profile.m_version; };
		QString	getProfileChapter()		{ return m_profile.m_chapter; };
		QString	getProfilePage()		{ return m_profile.m_page; };
		QString	getProfileLink()		{ return m_profile.m_link; };
		QString	getProfile( bool link = false );	
	
		bool isCommunicationObject();		// Objekt ist aus dem Kommunikationsprofil?
		bool isDeviceProfileObject();		// Objekt ist aus einem Geräteprofil?
		bool isManufacturerSpecificObject();// Objekt ist Hersteller-spezifisch?
		bool isSingleObject();				// Objekt ist eine VARiable?
		bool isMultipleObject();			// Objekt ist ein ARRAY oder ein RECORD?
		bool isStandardDataType();			// Objekt hat einen Basis-Datentyp?
		bool isComplexDataType();			// Objekt hat einen strukturierten Datentyp?
	
		int find( unsigned char subIndex );	// search object entry by sub-index (result -1 = none)
		int find( const char* name );		// search object entry by name (result -1 = none)

	protected:
		friend class CCANopenReader;		// access by class CCANopenReader allowed

		void setName( const char* name )	{ m_name = name; }
		void setClass( int code )			{ m_class = (TObjectClass)code; };
		void setDataType( int dataType )	{ m_dataType = (TObjectType)dataType; };
		void setCategory( int category )	{ m_category = (TObjectCategory)category; };
		void setProfileNumber( const char* number)		{ m_profile.m_number = number; };
		void setProfileVersion( const char* version)	{ m_profile.m_version = version; };
		void setProfileChapter( const char* chapter)	{ m_profile.m_chapter = chapter; };
		void setProfilePage( const char* page)			{ m_profile.m_page = page; };
		void setProfileLink( const char* link)			{ m_profile.m_link = link; };
	
	//	static member
	public:
		static QString getStateByIndex( TObjectState index );
		static QString getClassByIndex( TObjectClass index );
		static QString getDataTypeByIndex( TObjectType index );
		static QString getCategoryByIndex( TObjectCategory index );
		static int getStateByName( const char* name );
		static int getClassByName( const char* name );
		static int getDataTypeByName( const char* name );
		static int getCategoryByName( const char* name );
	
	//	revision control:
	public:
		static const char* version();
};

#endif // _CANopenObject_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
