/*	-- $Header$ --
 *
 *	projekt   :  CAN - Controller Area Network
 *
 *	purpose   :  Definitions and Options (CANopen-over-TCP/IP)
 *
 *	copyright :  (c) 2009, UV Software, Friedrichshafen
 *
 *	compiler  :  GCC - GNU C Compiler (Linux Kernel 2.6)
 *
 *	export    :  (see below)
 *
 *	includes  :  (none)
 *
 *	author    :  Uwe Vogt, UV Software, Friedrichshafen
 *
 *	e-mail    :  uwe.vogt@uv-software.de
 *
 *
 *	-----------  description  -----------------------------------------------
 *
 *	CANopen-over-TCP/IP Definitions and Options.
 *
 *
 *	-----------  history  ---------------------------------------------------
 *
 *	$Log$
 */

#ifndef __TCP_DEFS_H
#define __TCP_DEFS_H


/*  -----------  options  --------------------------------------------------
 */

#define _TOP_EVENT_QUEUE				/* Receiver Queue for Event-handling */


/*  -----------  defines  --------------------------------------------------
 */

#ifndef _TOP_DEFS						/* CANopen-over-TCP/IP Interfaces */

 #define CAN_REMOTE				(-1L)	/*   It´s a remote device */
 
 struct _top_param						/*   Installation parameters: */
 {
	unsigned long  addr;				/*     32-bit IPv4 address */
										/*     (network byte ordered) */
	unsigned short port;				/*     16-bit TCP or UDP port number */
										/*     (network byte ordered) */
 	/*int family = AF_INET;				//     Protocol family */
 	/*int type = SOCK_STREAM;			//     Communication semantics */
 	/*int protocol = 0;					//     Protocol to be used with the socket */

	struct _top_settings				/*     Settings for the gateway: */
	{
		unsigned char net;				/*       default network number */
		unsigned char node;				/*       default node-id */
		unsigned char master;			/*       node-id of the master */
	}	settings;
 };
 #define CANERR_TCPIP			(-20000)/*   socket error (variable 'errno' is set) */
#endif

/*  -----------  useful stuff  ---------------------------------------------
 */

/* *** **
typedef int					BOOL;
#define FALSE				0
#define TRUE				1
#define OK					0

typedef unsigned char		BYTE;
typedef unsigned short		WORD;
typedef unsigned long		DWORD;

typedef char				CHAR;
typedef short				SHORT;
typedef long				LONG;

typedef char*           	PSTR;
typedef char*           	NPSTR;

typedef char*           	LPSTR;
typedef const char*     	LPCSTR;
typedef BYTE*           	PBYTE;
typedef BYTE*           	LPBYTE;
typedef int*            	PINT;
typedef int*            	LPINT;
typedef WORD*           	PWORD;
typedef WORD*           	LPWORD;
typedef long*           	PLONG;
typedef long*           	LPLONG;
typedef DWORD*          	PDWORD;
typedef DWORD*          	LPDWORD;
typedef void*           	LPVOID;

#define BYTES2WORD(low,high)	(((unsigned int)((unsigned char)(low)))  << 0) | \
				(((unsigned int)((unsigned char)(high))) << 8)
#define WORDS2DWORD(low,high)	(((unsigned long)((unsigned int)(low)))  << 0) | \
				(((unsigned long)((unsigned int)(high))) << 16)
#define BYTES2DWORD(ll,lh,hl,hh)(((unsigned long)((unsigned char)(ll)))  << 0) | \
				(((unsigned long)((unsigned char)(lh))) << 8) | \
				(((unsigned long)((unsigned char)(hl))) << 16) | \
				(((unsigned long)((unsigned char)(hh))) << 24)

#define BYTEV2WORD(array)	(((unsigned int)((unsigned char)(array[0]))) << 0) | \
				(((unsigned int)((unsigned char)(array[1]))) << 8)
#define BYTEV2DWORD(array)	(((unsigned long)((unsigned char)(array[0]))) << 0) | \
				(((unsigned long)((unsigned char)(array[1]))) << 8) | \
				(((unsigned long)((unsigned char)(array[2]))) << 16) | \
				(((unsigned long)((unsigned char)(array[3]))) << 24)
** *** */

/*  -----------  types  ----------------------------------------------------
 */


/*  -----------  variables  ------------------------------------------------
 */


/*  -----------  prototypes  -----------------------------------------------
 */


#endif      // __CAN_DEFS_H

 /*	--------------------------------------------------------------------------
  *	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
  *	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
  *	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
  */
